@@ -45,11 +45,15 @@
 #include <unistd.h>
 #include <sys/wait.h>
 #include <sys/types.h>
+#ifdef __linux__
 #include <sys/syscall.h>
+#endif
+#include <pthread.h>
 #include <sys/utsname.h>
 #include <termios.h>
 #include <errno.h>
 #include <time.h>
+#include <ctype.h>
 
 #include "drmtest.h"
 #include "intel_chipset.h"
@@ -61,13 +65,17 @@
 #include "igt_core.h"
 #include "igt_aux.h"
 
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h>   /* for basename() on Solaris */
+#endif
+
 /**
  * SECTION:igt_core
  * @short_description: Core i-g-t testing support
  * @title: i-g-t core
  * @include: igt_core.h
  *
- * This libary implements the core of the i-g-t test support infrastructure.
+ * This library implements the core of the i-g-t test support infrastructure.
  * Main features are the subtest enumeration, cmdline option parsing helpers for
  * subtest handling and various helpers to structure testcases with subtests and
  * handle subtest test results.
@@ -141,9 +149,9 @@
  *   conditions, but instead such assumptions should be written in a declarative
  *   style.  Use one of the many macros which encapsulate i-g-t's implicit
  *   control flow.  Pick the most suitable one to have as much debug output as
- *   possible without polluting the code unecessarily. For example
+ *   possible without polluting the code unnecessarily. For example
  *   igt_assert_cmpint() for comparing integers or do_ioctl() for running ioctls
- *   and checking their results.  Feel free to add new ones to the libary or
+ *   and checking their results.  Feel free to add new ones to the library or
  *   wrap up a set of checks into a private function to further condense your
  *   test logic.
  *
@@ -197,10 +205,11 @@
  * run as non-root and doesn't require the i915 driver to be loaded (or any
  * intel gpu to be present). Then individual subtests can be run with
  * "--run-subtest". Usage help for tests with subtests can be obtained with the
- * "--help" commandline option.
+ * "--help" command line option.
  */
 
 static unsigned int exit_handler_count;
+const char *igt_interactive_debug;
 
 /* subtests helpers */
 static bool list_subtests = false;
@@ -223,10 +232,75 @@ bool test_child;
 enum {
  OPT_LIST_SUBTESTS,
  OPT_RUN_SUBTEST,
+ OPT_DESCRIPTION,
  OPT_DEBUG,
+ OPT_INTERACTIVE_DEBUG,
  OPT_HELP = 'h'
 };
 
+static int igt_exitcode = IGT_EXIT_SUCCESS;
+static const char *command_str;
+
+static char* igt_log_domain_filter;
+static struct {
+	char *entries[256];
+	uint8_t start, end;
+} log_buffer;
+static pthread_mutex_t log_buffer_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+static void _igt_log_buffer_append(char *line)
+{
+	pthread_mutex_lock(&log_buffer_mutex);
+
+	free(log_buffer.entries[log_buffer.end]);
+	log_buffer.entries[log_buffer.end] = line;
+	log_buffer.end++;
+	if (log_buffer.end == log_buffer.start)
+		log_buffer.start++;
+
+	pthread_mutex_unlock(&log_buffer_mutex);
+}
+
+static void _igt_log_buffer_reset(void)
+{
+	pthread_mutex_lock(&log_buffer_mutex);
+
+	log_buffer.start = log_buffer.end = 0;
+
+	pthread_mutex_unlock(&log_buffer_mutex);
+}
+
+static void _igt_log_buffer_dump(void)
+{
+	uint8_t i;
+
+	if (in_subtest)
+		fprintf(stderr, "Subtest %s failed.\n", in_subtest);
+	else
+		fprintf(stderr, "Test %s failed.\n", command_str);
+
+	if (log_buffer.start == log_buffer.end) {
+		fprintf(stderr, "No log.\n");
+		return;
+	}
+
+	pthread_mutex_lock(&log_buffer_mutex);
+	fprintf(stderr, "**** DEBUG ****\n");
+
+	i = log_buffer.start;
+	do {
+		char *last_line = log_buffer.entries[i];
+		fprintf(stderr, "%s", last_line);
+		i++;
+	} while (i != log_buffer.start && i != log_buffer.end);
+
+	/* reset the buffer */
+	log_buffer.start = log_buffer.end = 0;
+
+	fprintf(stderr, "****  END  ****\n");
+	pthread_mutex_unlock(&log_buffer_mutex);
+}
+
 __attribute__((format(printf, 1, 2)))
 static void kmsg(const char *format, ...)
 #define KERN_EMER	"<0>"
@@ -256,7 +330,9 @@ static void gettime(struct timespec *ts)
 {
 	memset(ts, 0, sizeof(*ts));
 
+#ifdef CLOCK_MONOTONIC_COARSE
 	if (clock_gettime(CLOCK_MONOTONIC_COARSE, ts))
+#endif
 		clock_gettime(CLOCK_MONOTONIC, ts);
 }
 
@@ -289,14 +365,83 @@ void __igt_fixture_end(void)
 	longjmp(igt_subtest_jmpbuf, 1);
 }
 
-bool igt_exit_called;
-static void check_igt_exit(int sig)
+/*
+ * Some of the IGT tests put quite a lot of pressure on memory and when
+ * running on Android they are sometimes killed by the Android low memory killer.
+ * This seems to be due to some incompatibility between the kswapd free memory
+ * targets and the way the lowmemorykiller assesses free memory.
+ * The low memory killer really isn't usefull in this context and has no
+ * interaction with the gpu driver that we are testing, so the following
+ * function is used to disable it by modifying one of its module parameters.
+ * We still have the normal linux oom killer to protect the kernel.
+ * Apparently it is also possible for the lowmemorykiller to get included
+ * in some linux distributions; so rather than check for Android we directly
+ * check for the existence of the module parameter we want to adjust.
+ *
+ * In future, if we can get the lowmemorykiller to play nicely then we can
+ * remove this hack.
+ */
+static void low_mem_killer_disable(bool disable)
 {
+	static const char* adj_fname="/sys/module/lowmemorykiller/parameters/adj";
+	static const char no_lowmem_killer[] = "9999";
+	int fd;
+	struct stat buf;
+	/* The following must persist across invocations */
+	static char prev_adj_scores[256];
+	static int adj_scores_len = 0;
+
+	/* capture the permissions bits for the lowmemkiller adj pseudo-file.
+	 * Bail out if the stat fails; it probably means that there is no
+	 * lowmemorykiller, but in any case we're doomed. */
+	if (stat(adj_fname, &buf)) {
+		igt_assert(errno == ENOENT);
+		return;
+	}
+
+	/* make sure the file can be read/written - by default it is write-only */
+	chmod(adj_fname, S_IRUSR | S_IWUSR);
+
+	if (disable) {
+		/* read the current oom adj parameters for lowmemorykiller */
+		fd = open(adj_fname, O_RDWR);
+		igt_assert(fd != -1);
+		adj_scores_len = read(fd, (void*)prev_adj_scores, 255);
+		igt_assert(adj_scores_len > 0);
+
+		/* writing 9999 to this module parameter effectively diables the
+		 * low memory killer. This is not a real file, so we dont need to
+		 * seek to the start or truncate it */
+		write(fd, no_lowmem_killer, sizeof(no_lowmem_killer));
+		close(fd);
+	} else {
+		/* just re-enstate the original settings */
+		fd = open(adj_fname, O_WRONLY);
+		igt_assert(fd != -1);
+		write(fd, prev_adj_scores, adj_scores_len);
+		close(fd);
+	}
+
+	/* re-enstate the file permissions */
+	chmod(adj_fname, buf.st_mode);
+}
+
+bool igt_exit_called;
+static void common_exit_handler(int sig)
+{
+	low_mem_killer_disable(false);
+
 	/* When not killed by a signal check that igt_exit() has been properly
 	 * called. */
 	assert(sig != 0 || igt_exit_called);
 }
 
+static void print_test_description(void)
+{
+	if (&__igt_test_description)
+		printf("%s\n", __igt_test_description);
+}
+
 static void print_version(void)
 {
 	struct utsname uts;
@@ -311,21 +456,22 @@ static void print_version(void)
 		uts.sysname, uts.release, uts.machine);
 }
 
-static const char *command_str;
-
 static void print_usage(const char *help_str, bool output_on_stderr)
 {
 	FILE *f = output_on_stderr ? stderr : stdout;
 
-	fprintf(f, "Usage: %s [OPTIONS]\n"
-		   "  --list-subtests\n"
+	fprintf(f, "Usage: %s [OPTIONS]\n", command_str);
+	fprintf(f, "  --list-subtests\n"
 		   "  --run-subtest <pattern>\n"
-		   "  --debug\n"
-		   "  --help\n", command_str);
+		   "  --debug[=log-domain]\n"
+		   "  --interactive-debug[=domain]\n"
+		   "  --help-description\n"
+		   "  --help\n");
 	if (help_str)
 		fprintf(f, "%s\n", help_str);
 }
 
+
 static void oom_adjust_for_doom(void)
 {
 	int fd;
@@ -334,9 +480,12 @@ static void oom_adjust_for_doom(void)
 	fd = open("/proc/self/oom_score_adj", O_WRONLY);
 	igt_assert(fd != -1);
 	igt_assert(write(fd, always_kill, sizeof(always_kill)) == sizeof(always_kill));
+	close(fd);
+
+	low_mem_killer_disable(true);
 }
 
-static int common_init(int argc, char **argv,
+static int common_init(int *argc, char **argv,
 		       const char *extra_short_opts,
 		       struct option *extra_long_opts,
 		       const char *help_str,
@@ -346,7 +495,9 @@ static int common_init(int argc, char **argv,
 	static struct option long_options[] = {
 		{"list-subtests", 0, 0, OPT_LIST_SUBTESTS},
 		{"run-subtest", 1, 0, OPT_RUN_SUBTEST},
-		{"debug", 0, 0, OPT_DEBUG},
+		{"help-description", 0, 0, OPT_DESCRIPTION},
+		{"debug", optional_argument, 0, OPT_DEBUG},
+		{"interactive-debug", optional_argument, 0, OPT_INTERACTIVE_DEBUG},
 		{"help", 0, 0, OPT_HELP},
 		{0, 0, 0, 0}
 	};
@@ -429,11 +580,19 @@ static int common_init(int argc, char **argv,
 		       std_short_opts);
 	assert(ret >= 0);
 
-	while ((c = getopt_long(argc, argv, short_opts, combined_opts,
+	while ((c = getopt_long(*argc, argv, short_opts, combined_opts,
 			       &option_index)) != -1) {
 		switch(c) {
+		case OPT_INTERACTIVE_DEBUG:
+			if (optarg && strlen(optarg) > 0)
+				igt_interactive_debug = strdup(optarg);
+			else
+				igt_interactive_debug = "all";
+			break;
 		case OPT_DEBUG:
 			igt_log_level = IGT_LOG_DEBUG;
+			if (optarg && strlen(optarg) > 0)
+				igt_log_domain_filter = strdup(optarg);
 			break;
 		case OPT_LIST_SUBTESTS:
 			if (!run_single_subtest)
@@ -443,6 +602,10 @@ static int common_init(int argc, char **argv,
 			if (!list_subtests)
 				run_single_subtest = strdup(optarg);
 			break;
+		case OPT_DESCRIPTION:
+			print_test_description();
+			ret = -1;
+			goto out;
 		case OPT_HELP:
 			print_usage(help_str, false);
 			ret = -1;
@@ -484,6 +647,17 @@ out:
 		oom_adjust_for_doom();
 	}
 
+	/* install exit handler, to ensure we clean up */
+	igt_install_exit_handler(common_exit_handler);
+
+	if (!test_with_subtests)
+		gettime(&subtest_time);
+
+	for (i = 0; (optind + i) < *argc; i++)
+		argv[i + 1] = argv[optind + i];
+
+	*argc = *argc - optind + 1;
+
 	return ret;
 }
 
@@ -507,7 +681,7 @@ out:
  *
  * Returns: Forwards any option parsing errors from getopt_long.
  */
-int igt_subtest_init_parse_opts(int argc, char **argv,
+int igt_subtest_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
 				struct option *extra_long_opts,
 				const char *help_str,
@@ -518,47 +692,12 @@ int igt_subtest_init_parse_opts(int argc, char **argv,
 	test_with_subtests = true;
 	ret = common_init(argc, argv, extra_short_opts, extra_long_opts,
 			  help_str, extra_opt_handler);
-	igt_install_exit_handler(check_igt_exit);
 
 	return ret;
 }
 
 enum igt_log_level igt_log_level = IGT_LOG_INFO;
 
-/**
- * igt_subtest_init:
- * @argc: argc from the test's main()
- * @argv: argv from the test's main()
- *
- * This initializes the for tests with subtests without the need for additional
- * cmdline options. It is just a simplified version of
- * igt_subtest_init_parse_opts().
- *
- * If there's not a reason to the contrary it's less error prone to just use an
- * #igt_main block instead of stitching the tests's main() function together
- * manually.
- */
-void igt_subtest_init(int argc, char **argv)
-{
-	igt_subtest_init_parse_opts(argc, argv, NULL, NULL, NULL, NULL);
-}
-
-/**
- * igt_simple_init:
- * @argc: argc from the test's main()
- * @argv: argv from the test's main()
- *
- * This initializes a simple test without any support for subtests.
- *
- * If there's not a reason to the contrary it's less error prone to just use an
- * #igt_simple_main block instead of stitching the tests's main() function together
- * manually.
- */
-void igt_simple_init(int argc, char **argv)
-{
-	common_init(argc, argv, NULL, NULL, NULL, NULL);
-}
-
 /**
  * igt_simple_init_parse_opts:
  * @argc: argc from the test's main()
@@ -571,7 +710,7 @@ void igt_simple_init(int argc, char **argv)
  * This initializes a simple test without any support for subtests and allows
  * an arbitrary set of additional options.
  */
-void igt_simple_init_parse_opts(int argc, char **argv,
+void igt_simple_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
 				struct option *extra_long_opts,
 				const char *help_str,
@@ -588,8 +727,20 @@ void igt_simple_init_parse_opts(int argc, char **argv,
  */
 bool __igt_run_subtest(const char *subtest_name)
 {
+	int i;
+
 	assert(!in_subtest);
 	assert(!in_fixture);
+	assert(test_with_subtests);
+
+	/* check the subtest name only contains a-z, A-Z, 0-9, '-' and '_' */
+	for (i = 0; subtest_name[i] != '\0'; i++)
+		if (subtest_name[i] != '_' && subtest_name[i] != '-'
+		    && !isalnum(subtest_name[i])) {
+			igt_critical("Invalid subtest name \"%s\".\n",
+				     subtest_name);
+			igt_exit();
+		}
 
 	if (list_subtests) {
 		printf("%s\n", subtest_name);
@@ -611,6 +762,9 @@ bool __igt_run_subtest(const char *subtest_name)
 	}
 
 	kmsg(KERN_INFO "%s: starting subtest %s\n", command_str, subtest_name);
+	igt_debug("Starting subtest: %s\n", subtest_name);
+
+	_igt_log_buffer_reset();
 
 	gettime(&subtest_time);
 	return (in_subtest = subtest_name);
@@ -641,7 +795,6 @@ bool igt_only_list_subtests(void)
 static bool skipped_one = false;
 static bool succeeded_one = false;
 static bool failed_one = false;
-static int igt_exitcode;
 
 static void exit_subtest(const char *) __attribute__((noreturn));
 static void exit_subtest(const char *result)
@@ -692,7 +845,8 @@ void igt_skip(const char *f, ...)
 		assert(in_fixture);
 		__igt_fixture_end();
 	} else {
-		exit(IGT_EXIT_SKIP);
+		igt_exitcode = IGT_EXIT_SKIP;
+		igt_exit();
 	}
 }
 
@@ -733,7 +887,7 @@ void __igt_skip_check(const char *file, const int line,
 /**
  * igt_success:
  *
- * Complete a (subtest) as successfull
+ * Complete a (subtest) as successful
  *
  * This bails out of a subtests and marks it as successful. For global tests it
  * it won't bail out of anything.
@@ -773,6 +927,8 @@ void igt_fail(int exitcode)
 	if (test_child)
 		exit(exitcode);
 
+	_igt_log_buffer_dump();
+
 	if (in_subtest) {
 		if (exitcode == IGT_EXIT_TIMEOUT)
 			exit_subtest("TIMEOUT");
@@ -786,7 +942,7 @@ void igt_fail(int exitcode)
 			__igt_fixture_end();
 		}
 
-		exit(exitcode);
+		igt_exit();
 	}
 }
 
@@ -799,27 +955,24 @@ static bool run_under_gdb(void)
 		strncmp(basename(buf), "gdb", 3) == 0);
 }
 
-void __igt_fail_assert(int exitcode, const char *file,
+void __igt_fail_assert(int exitcode, const char *domain, const char *file,
 		       const int line, const char *func, const char *assertion,
 		       const char *f, ...)
 {
 	va_list args;
 	int err = errno;
-	char *err_str = NULL;
 
+	igt_log(domain, IGT_LOG_CRITICAL,
+		"Test assertion failure function %s, file %s:%i:\n", func, file,
+		line);
+	igt_log(domain, IGT_LOG_CRITICAL, "Failed assertion: %s\n", assertion);
 	if (err)
-		asprintf(&err_str, "Last errno: %i, %s\n", err, strerror(err));
-
-	printf("Test assertion failure function %s, file %s:%i:\n"
-	       "Failed assertion: %s\n"
-	       "%s",
-	       func, file, line, assertion, err_str ?: "");
-
-	free(err_str);
+		igt_log(domain, IGT_LOG_CRITICAL, "Last errno: %i, %s\n", err,
+			strerror(err));
 
 	if (f) {
 		va_start(args, f);
-		vprintf(f, args);
+		igt_vlog(domain, IGT_LOG_CRITICAL, f, args);
 		va_end(args);
 	}
 
@@ -838,10 +991,9 @@ void __igt_fail_assert(int exitcode, const char *file,
  * everything has worked out. For subtests it also checks that at least one
  * subtest has been run (save when only listing subtests.
  *
- * It is an error to normally exit a test with subtests without calling
- * igt_exit() - without it the result reporting will be wrong. To avoid such
- * issues it is highly recommended to use #igt_main instead of a hand-rolled
- * main() function.
+ * It is an error to normally exit a test calling igt_exit() - without it the
+ * result reporting will be wrong. To avoid such issues it is highly recommended
+ * to use #igt_main or #igt_simple_main instead of a hand-rolled main() function.
  */
 void igt_exit(void)
 {
@@ -856,8 +1008,36 @@ void igt_exit(void)
 	if (igt_only_list_subtests())
 		exit(IGT_EXIT_SUCCESS);
 
-	if (!test_with_subtests)
-		exit(IGT_EXIT_SUCCESS);
+	kmsg(KERN_INFO "%s: exiting, ret=%d\n", command_str, igt_exitcode);
+	igt_debug("Exiting with status code %d\n", igt_exitcode);
+
+	if (!test_with_subtests) {
+		struct timespec now;
+		double elapsed;
+		const char *result;
+
+		gettime(&now);
+		elapsed = now.tv_sec - subtest_time.tv_sec;
+		elapsed += (now.tv_nsec - subtest_time.tv_nsec) * 1e-9;
+
+		switch (igt_exitcode) {
+			case IGT_EXIT_SUCCESS:
+				result = "SUCCESS";
+				break;
+			case IGT_EXIT_TIMEOUT:
+				result = "TIMEOUT";
+				break;
+			case IGT_EXIT_SKIP:
+				result = "SKIP";
+				break;
+			default:
+				result = "FAIL";
+		}
+
+
+		printf("%s (%.3fs)\n", result, elapsed);
+		exit(igt_exitcode);
+	}
 
 	/* Calling this without calling one of the above is a failure */
 	assert(skipped_one || succeeded_one || failed_one);
@@ -928,6 +1108,10 @@ bool __igt_fork_helper(struct igt_helper_process *proc)
 	 */
 	tmp_count = exit_handler_count;
 	exit_handler_count = 0;
+
+	/* ensure any buffers are flushed before fork */
+	fflush(NULL);
+
 	switch (pid = fork()) {
 	case -1:
 		exit_handler_count = tmp_count;
@@ -1019,6 +1203,9 @@ bool __igt_fork(void)
 		igt_assert(test_children);
 	}
 
+	/* ensure any buffers are flushed before fork */
+	fflush(NULL);
+
 	switch (test_children[num_test_children++] = fork()) {
 	case -1:
 		igt_assert(0);
@@ -1041,9 +1228,9 @@ bool __igt_fork(void)
  * Wait for all children forked with igt_fork.
  *
  * The magic here is that exit codes from children will be correctly propagated
- * to the main thread, including the relevant exitcode if a child thread failed.
- * Of course if multiple children failed with differen exitcodes the resulting
- * exitcode will be non-deterministic.
+ * to the main thread, including the relevant exit code if a child thread failed.
+ * Of course if multiple children failed with different exit codes the resulting
+ * exit code will be non-deterministic.
  *
  * Note that igt_skip() will not be forwarded, feature tests need to be done
  * before spawning threads with igt_fork().
@@ -1102,6 +1289,10 @@ void igt_waitchildren(void)
 #define MAX_SIGNALS		32
 #define MAX_EXIT_HANDLERS	10
 
+#ifndef HAVE_SIGHANDLER_T
+typedef void (*sighandler_t)(int);
+#endif
+
 static struct {
 	sighandler_t handler;
 	bool installed;
@@ -1110,8 +1301,10 @@ static struct {
 static igt_exit_handler_t exit_handler_fn[MAX_EXIT_HANDLERS];
 static bool exit_handler_disabled;
 static sigset_t saved_sig_mask;
-static const int handled_signals[] =
-	{ SIGINT, SIGHUP, SIGTERM, SIGQUIT, SIGPIPE, SIGABRT, SIGSEGV, SIGBUS };
+#define SIGDEF(x) { x, #x, sizeof(#x) - 1 }
+static const struct { int number; const char *name; size_t name_len; } handled_signals[] =
+	{ SIGDEF(SIGINT), SIGDEF(SIGHUP), SIGDEF(SIGTERM), SIGDEF(SIGQUIT),
+	  SIGDEF(SIGPIPE), SIGDEF(SIGABRT), SIGDEF(SIGSEGV), SIGDEF(SIGBUS) };
 
 static int install_sig_handler(int sig_num, sighandler_t handler)
 {
@@ -1164,21 +1357,40 @@ static void igt_atexit_handler(void)
 
 static void fatal_sig_handler(int sig)
 {
-	pid_t pid, tid;
+	int i;
 
 	restore_all_sig_handler();
 
+	for (i = 0; i < ARRAY_SIZE(handled_signals); i++) {
+		if (handled_signals[i].number == sig) {
+			write(STDERR_FILENO, "Received signal ", 16);
+			write(STDERR_FILENO, handled_signals[i].name,
+			      handled_signals[i].name_len);
+			write(STDERR_FILENO, ".\n", 2);
+			break;
+		}
+	}
+
 	/*
 	 * exit_handler_disabled is always false here, since when we set it
 	 * we also block signals.
 	 */
 	call_exit_handlers(sig);
 
+	{
+#ifdef __linux__
 	/* Workaround cached PID and TID races on glibc and Bionic libc. */
-	pid = syscall(SYS_getpid);
-	tid = syscall(SYS_gettid);
+		pid_t pid = syscall(SYS_getpid);
+		pid_t tid = syscall(SYS_gettid);
 
-	syscall(SYS_tgkill, pid, tid, sig);
+		syscall(SYS_tgkill, pid, tid, sig);
+#else
+		pthread_t tid = pthread_self();
+		union sigval value = { .sival_ptr = NULL };
+
+		pthread_sigqueue(tid, sig, value);
+#endif
+        }
 }
 
 /**
@@ -1215,7 +1427,7 @@ void igt_install_exit_handler(igt_exit_handler_t fn)
 		return;
 
 	for (i = 0; i < ARRAY_SIZE(handled_signals); i++) {
-		if (install_sig_handler(handled_signals[i],
+		if (install_sig_handler(handled_signals[i].number,
 					fatal_sig_handler))
 			goto err;
 	}
@@ -1247,7 +1459,7 @@ void igt_disable_exit_handler(void)
 
 	sigemptyset(&set);
 	for (i = 0; i < ARRAY_SIZE(handled_signals); i++)
-		sigaddset(&set, handled_signals[i]);
+		sigaddset(&set, handled_signals[i].number);
 
 	if (sigprocmask(SIG_BLOCK, &set, &saved_sig_mask)) {
 		perror("sigprocmask");
@@ -1282,8 +1494,8 @@ void igt_enable_exit_handler(void)
  * igt_run_in_simulation:
  *
  * This function can be used to select a reduced test set when running in
- * simulation enviroments. This i-g-t mode is selected by setting the
- * INTEL_SIMULATION enviroment variable to 1.
+ * simulation environments. This i-g-t mode is selected by setting the
+ * INTEL_SIMULATION environment variable to 1.
  *
  * Returns: True when run in simulation mode, false otherwise.
  */
@@ -1325,6 +1537,7 @@ void igt_skip_on_simulation(void)
 
 /**
  * igt_log:
+ * @domain: the log domain, or NULL for no domain
  * @level: #igt_log_level
  * @format: format string
  * @...: optional arguments used in the format string
@@ -1334,34 +1547,24 @@ void igt_skip_on_simulation(void)
  * to stderr and the test runner should treat this as an intermediate result
  * between SUCESS and FAILURE.
  *
- * The log level can be set through the IGT_LOG_LEVEL enviroment variable with
- * values "debug", "info", "warn" and "none". By default verbose debug message
- * are disabled. "none" completely disables all output and is not recommended
- * since crucial issues only reported at the IGT_LOG_WARN level are ignored.
+ * The log level can be set through the IGT_LOG_LEVEL environment variable with
+ * values "debug", "info", "warn", "critical" and "none". By default verbose
+ * debug message are disabled. "none" completely disables all output and is not
+ * recommended since crucial issues only reported at the IGT_LOG_WARN level are
+ * ignored.
  */
-void igt_log(enum igt_log_level level, const char *format, ...)
+void igt_log(const char *domain, enum igt_log_level level, const char *format, ...)
 {
 	va_list args;
 
-	assert(format);
-
-	if (list_subtests)
-		return;
-
-	if (igt_log_level > level)
-		return;
-
 	va_start(args, format);
-	if (level == IGT_LOG_WARN) {
-		fflush(stdout);
-		vfprintf(stderr, format, args);
-	} else
-		vprintf(format, args);
+	igt_vlog(domain, level, format, args);
 	va_end(args);
 }
 
 /**
  * igt_vlog:
+ * @domain: the log domain, or NULL for no domain
  * @level: #igt_log_level
  * @format: format string
  * @args: variable arguments lists
@@ -1373,28 +1576,85 @@ void igt_log(enum igt_log_level level, const char *format, ...)
  * If there is no need to wrap up a vararg list in the caller it is simpler to
  * just use igt_log().
  */
-void igt_vlog(enum igt_log_level level, const char *format, va_list args)
+void igt_vlog(const char *domain, enum igt_log_level level, const char *format, va_list args)
 {
+	FILE *file;
+	char *line, *formatted_line;
+	const char *program_name;
+	const char *igt_log_level_str[] = {
+		"DEBUG",
+		"INFO",
+		"WARNING",
+		"CRITICAL",
+		"NONE"
+	};
+	static bool line_continuation = false;
+
 	assert(format);
 
-	if (list_subtests)
+#ifdef __GLIBC__
+	program_name = program_invocation_short_name;
+#else
+	program_name = command_str;
+#endif
+
+	if (list_subtests && level <= IGT_LOG_WARN)
 		return;
 
+	if (vasprintf(&line, format, args) == -1)
+		return;
+
+	if (line_continuation) {
+		formatted_line = strdup(line);
+		if (!formatted_line)
+			goto out;
+	} else if (asprintf(&formatted_line, "(%s:%d) %s%s%s: %s", program_name,
+		     getpid(), (domain) ? domain : "", (domain) ? "-" : "",
+		     igt_log_level_str[level], line) == -1) {
+		goto out;
+	}
+
+	line_continuation = line[strlen(line)] != '\n';
+
+	/* append log buffer */
+	_igt_log_buffer_append(formatted_line);
+
+	/* check print log level */
 	if (igt_log_level > level)
-		return;
+		goto out;
 
-	if (level == IGT_LOG_WARN) {
+	/* check domain filter */
+	if (igt_log_domain_filter) {
+		/* if null domain and filter is not "application", return */
+		if (!domain && strcmp(igt_log_domain_filter, "application"))
+			goto out;
+		/* else if domain and filter do not match, return */
+		else if (domain && strcmp(igt_log_domain_filter, domain))
+			goto out;
+	}
+
+	/* use stderr for warning messages and above */
+	if (level >= IGT_LOG_WARN) {
+		file = stderr;
 		fflush(stdout);
-		vfprintf(stderr, format, args);
-	} else
-		vprintf(format, args);
+	}
+	else
+		file = stdout;
+
+	/* prepend all except information messages with process, domain and log
+	 * level information */
+	if (level != IGT_LOG_INFO)
+		fwrite(formatted_line, sizeof(char), strlen(formatted_line),
+		       file);
+	else
+		fwrite(line, sizeof(char), strlen(line), file);
+
+out:
+	free(line);
 }
 
 static void igt_alarm_handler(int signal)
 {
-	/* subsequent tests are skipped */
-	skip_subtests_henceforth = SKIP;
-
 	/* exit with timeout status */
 	igt_fail(IGT_EXIT_TIMEOUT);
 }
@@ -1403,11 +1663,13 @@ static void igt_alarm_handler(int signal)
  * igt_set_timeout:
  * @seconds: number of seconds before timeout
  *
- * Stop the current test and skip any subsequent tests after the specified
- * number of seconds have elapsed. The test will exit with #IGT_EXIT_TIMEOUT
- * status. Any previous timer is cancelled and no timeout is scheduled if
- * @seconds is zero.
+ * Fail a test and exit with #IGT_EXIT_TIMEOUT status after the specified
+ * number of seconds have elapsed. If the current test has subtests and the
+ * timeout occurs outside a subtest, subsequent subtests will be skipped and
+ * marked as failed.
  *
+ * Any previous timer is cancelled and no timeout is scheduled if @seconds is
+ * zero.
  */
 void igt_set_timeout(unsigned int seconds)
 {
