@@ -36,6 +36,24 @@
 #include <string.h>
 #include <sys/types.h>
 #include <stdarg.h>
+#include <getopt.h>
+
+#ifndef IGT_LOG_DOMAIN
+#define IGT_LOG_DOMAIN (NULL)
+#endif
+
+
+extern const char* __igt_test_description __attribute__((weak));
+
+/**
+ * IGT_TEST_DESCRIPTION:
+ * @str: description string
+ *
+ * Defines a description for a test. This is used as the output for the
+ * "--help-description" option and is also included in the generated
+ * documentation.
+ */
+#define IGT_TEST_DESCRIPTION(str) const char* __igt_test_description = str
 
 /**
  * IGT_EXIT_TIMEOUT:
@@ -58,6 +76,13 @@
  */
 #define IGT_EXIT_SUCCESS 0
 
+/**
+ * IGT_EXIT_INVALID
+ *
+ * Exit status indicating an invalid option or subtest was specified
+ */
+#define IGT_EXIT_INVALID 79
+
 
 bool __igt_fixture(void);
 void __igt_fixture_complete(void);
@@ -81,17 +106,32 @@ void __igt_fixture_end(void) __attribute__((noreturn));
 
 /* subtest infrastructure */
 jmp_buf igt_subtest_jmpbuf;
-void igt_subtest_init(int argc, char **argv);
 typedef int (*igt_opt_handler_t)(int opt, int opt_index);
 #ifndef __GTK_DOC_IGNORE__ /* gtkdoc wants to document this forward decl */
 struct option;
 #endif
-int igt_subtest_init_parse_opts(int argc, char **argv,
+int igt_subtest_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
 				struct option *extra_long_opts,
 				const char *help_str,
 				igt_opt_handler_t extra_opt_handler);
 
+
+/**
+ * igt_subtest_init:
+ * @argc: argc from the test's main()
+ * @argv: argv from the test's main()
+ *
+ * This initializes the for tests with subtests without the need for additional
+ * cmdline options. It is just a simplified version of
+ * igt_subtest_init_parse_opts().
+ *
+ * If there's not a reason to the contrary it's less error prone to just use an
+ * #igt_main block instead of stitching the test's main() function together
+ * manually.
+ */
+#define igt_subtest_init(argc, argv) igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL);
+
 bool __igt_run_subtest(const char *subtest_name);
 #define __igt_tokencat2(x, y) x ## y
 
@@ -110,7 +150,7 @@ bool __igt_run_subtest(const char *subtest_name);
  * @name: name of the subtest
  *
  * This is a magic control flow block which denotes a subtest code block. Within
- * that codeblock igt_skip|success will only bail out of the subtest. The _f
+ * that code block igt_skip|success will only bail out of the subtest. The _f
  * variant accepts a printf format string, which is useful for constructing
  * combinatorial tests.
  *
@@ -132,7 +172,7 @@ bool __igt_run_subtest(const char *subtest_name);
  * @...: format string and optional arguments
  *
  * This is a magic control flow block which denotes a subtest code block. Within
- * that codeblock igt_skip|success will only bail out of the subtest. The _f
+ * that code block igt_skip|success will only bail out of the subtest. The _f
  * variant accepts a printf format string, which is useful for constructing
  * combinatorial tests.
  *
@@ -155,18 +195,31 @@ bool igt_only_list_subtests(void);
 #define igt_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
-		igt_subtest_init(argc, argv); \
+		igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
 		igt_exit(); \
 	} \
 	static void igt_tokencat(__real_main, __LINE__)(void) \
 
+
+void igt_simple_init_parse_opts(int *argc, char **argv,
+				const char *extra_short_opts,
+				struct option *extra_long_opts,
+				const char *help_str,
+				igt_opt_handler_t extra_opt_handler);
+
 /**
  * igt_simple_init:
+ * @argc: argc from the test's main()
+ * @argv: argv from the test's main()
  *
- * Init for simple tests without subtests
+ * This initializes a simple test without any support for subtests.
+ *
+ * If there's not a reason to the contrary it's less error prone to just use an
+ * #igt_simple_main block instead of stitching the test's main() function together
+ * manually.
  */
-void igt_simple_init(void);
+#define igt_simple_init(argc, argv) igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL);
 
 /**
  * igt_simple_main:
@@ -178,9 +231,9 @@ void igt_simple_init(void);
 #define igt_simple_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
-		igt_simple_init(); \
+		igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
-		exit(0); \
+		igt_exit(); \
 	} \
 	static void igt_tokencat(__real_main, __LINE__)(void) \
 
@@ -190,11 +243,13 @@ __attribute__((format(printf, 5, 6)))
 void __igt_skip_check(const char *file, const int line,
 		      const char *func, const char *check,
 		      const char *format, ...) __attribute__((noreturn));
+#define igt_skip_check(E, F...) \
+	__igt_skip_check(__FILE__, __LINE__, __func__, E, F)
 void igt_success(void);
 
 void igt_fail(int exitcode) __attribute__((noreturn));
-__attribute__((format(printf, 6, 7)))
-void __igt_fail_assert(int exitcode, const char *file,
+__attribute__((format(printf, 7, 8)))
+void __igt_fail_assert(int exitcode, const char *domain, const char *file,
 		       const int line, const char *func, const char *assertion,
 		       const char *format, ...)
 	__attribute__((noreturn));
@@ -204,13 +259,13 @@ void igt_exit(void) __attribute__((noreturn));
  * igt_assert:
  * @expr: condition to test
  *
- * Fails (sub-)test if the condition is not met
+ * Fails (sub-)test if the condition is not met.
  *
  * Should be used everywhere where a test checks results.
  */
 #define igt_assert(expr) \
 	do { if (!(expr)) \
-		__igt_fail_assert(99, __FILE__, __LINE__, __func__, #expr , NULL); \
+		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, #expr , NULL); \
 	} while (0)
 
 /**
@@ -218,7 +273,7 @@ void igt_exit(void) __attribute__((noreturn));
  * @expr: condition to test
  * @...: format string and optional arguments
  *
- * Fails (sub-)test if the condition is not met
+ * Fails (sub-)test if the condition is not met.
  *
  * Should be used everywhere where a test checks results.
  *
@@ -227,13 +282,38 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert_f(expr, f...) \
 	do { if (!(expr)) \
-		__igt_fail_assert(99, __FILE__, __LINE__, __func__, #expr , f); \
+		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, #expr , f); \
 	} while (0)
 
+/**
+ * igt_fail_on:
+ * @expr: condition to test
+ *
+ * Fails (sub-)test if the condition is met.
+ *
+ * Should be used everywhere where a test checks results.
+ */
+#define igt_fail_on(expr) igt_assert(!(expr))
+
+/**
+ * igt_fail_on_f:
+ * @expr: condition to test
+ * @...: format string and optional arguments
+ *
+ * Fails (sub-)test if the condition is met.
+ *
+ * Should be used everywhere where a test checks results.
+ *
+ * In addition to the plain igt_assert() helper this allows to print additional
+ * information to help debugging test failures.
+ */
+#define igt_fail_on_f(expr, f...) igt_assert_f(!(expr), f)
+
 /**
  * igt_assert_cmpint:
  * @n1: first value
  * @cmp: compare operator
+ * @ncmp: negated version of @cmp
  * @n2: second value
  *
  * Fails (sub-)test if the condition is not met
@@ -243,15 +323,77 @@ void igt_exit(void) __attribute__((noreturn));
  * Like igt_assert(), but displays the values being compared on failure instead
  * of simply printing the stringified expression.
  */
-#define igt_assert_cmpint(n1, cmp, n2) \
+#define igt_assert_cmpint(n1, cmp, ncmp, n2) \
 	do { \
 		int __n1 = (n1), __n2 = (n2); \
 		if (__n1 cmp __n2) ; else \
-		__igt_fail_assert(99, __FILE__, __LINE__, __func__, \
+		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
 				  #n1 " " #cmp " " #n2, \
-				  "error: %d %s %d\n", __n1, #cmp, __n2); \
+				  "error: %d " #ncmp " %d\n", __n1, __n2); \
 	} while (0)
 
+#define igt_assert_cmpuint(n1, cmp, ncmp, n2) \
+	do { \
+		uint32_t __n1 = (n1), __n2 = (n2); \
+		if (__n1 cmp __n2) ; else \
+		__igt_fail_assert(99, IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+				  #n1 " " #cmp " " #n2, \
+				  "error: %#x " #ncmp " %#x\n", __n1, __n2); \
+	} while (0)
+
+/**
+ * igt_assert_eq:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Fails (sub-)test if the two integers are not equal. Beware that for now this
+ * only works on integers.
+ *
+ * Like igt_assert(), but displays the values being compared on failure instead
+ * of simply printing the stringified expression.
+ */
+#define igt_assert_eq(n1, n2) igt_assert_cmpint(n1, ==, !=, n2)
+#define igt_assert_eq_u32(n1, n2) igt_assert_cmpuint(n1, ==, !=, n2)
+
+/**
+ * igt_assert_neq:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Fails (sub-)test if the two integers are equal. Beware that for now this
+ * only works on integers.
+ *
+ * Like igt_assert(), but displays the values being compared on failure instead
+ * of simply printing the stringified expression.
+ */
+#define igt_assert_neq(n1, n2) igt_assert_cmpint(n1, !=, ==, n2)
+
+/**
+ * igt_assert_lte:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Fails (sub-)test if the second integers is greater than the first.
+ * Beware that for now this only works on integers.
+ *
+ * Like igt_assert(), but displays the values being compared on failure instead
+ * of simply printing the stringified expression.
+ */
+#define igt_assert_lte(n1, n2) igt_assert_cmpint(n1, <=, >, n2)
+
+/**
+ * igt_assert_lt:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Fails (sub-)test if the second integers is strictly smaller than the first.
+ * Beware that for now this only works on integers.
+ *
+ * Like igt_assert(), but displays the values being compared on failure instead
+ * of simply printing the stringified expression.
+ */
+#define igt_assert_lt(n1, n2) igt_assert_cmpint(n1, <, >=, n2)
+
 /**
  * igt_require:
  * @expr: condition to test
@@ -262,7 +404,10 @@ void igt_exit(void) __attribute__((noreturn));
  * skipping. This is useful to streamline the skip logic since it allows for a more flat
  * code control flow, similar to igt_assert()
  */
-#define igt_require(expr) igt_skip_on(!(expr))
+#define igt_require(expr) do { \
+	if (!(expr)) igt_skip_check(#expr , NULL); \
+	else igt_debug("Test requirement passed: "#expr"\n"); \
+} while (0)
 
 /**
  * igt_skip_on:
@@ -274,10 +419,10 @@ void igt_exit(void) __attribute__((noreturn));
  * skipping. This is useful to streamline the skip logic since it allows for a more flat
  * code control flow, similar to igt_assert()
  */
-#define igt_skip_on(expr) \
-	do { if ((expr)) \
-		__igt_skip_check(__FILE__, __LINE__, __func__, #expr , NULL); \
-	} while (0)
+#define igt_skip_on(expr) do { \
+	if ((expr)) igt_skip_check("!(" #expr ")" , NULL); \
+	else igt_debug("Test requirement passed: !("#expr")\n"); \
+} while (0)
 
 /**
  * igt_require_f:
@@ -293,7 +438,10 @@ void igt_exit(void) __attribute__((noreturn));
  * In addition to the plain igt_require() helper this allows to print additional
  * information to help debugging test failures.
  */
-#define igt_require_f(expr, f...) igt_skip_on_f(!(expr), f)
+#define igt_require_f(expr, f...) do { \
+	if (!(expr)) igt_skip_check(#expr , f); \
+	else igt_debug("Test requirement passed: "#expr"\n"); \
+} while (0)
 
 /**
  * igt_skip_on_f:
@@ -309,10 +457,10 @@ void igt_exit(void) __attribute__((noreturn));
  * In addition to the plain igt_skip_on() helper this allows to print additional
  * information to help debugging test failures.
  */
-#define igt_skip_on_f(expr, f...) \
-	do { if ((expr)) \
-		__igt_skip_check(__FILE__, __LINE__, __func__, #expr , f); \
-	} while (0)
+#define igt_skip_on_f(expr, f...) do { \
+	if ((expr)) igt_skip_check("!("#expr")", f); \
+	else igt_debug("Test requirement passed: !("#expr")\n"); \
+} while (0)
 
 /* fork support code */
 bool __igt_fork(void);
@@ -338,9 +486,9 @@ bool __igt_fork(void);
 void igt_waitchildren(void);
 
 /**
- * igt_helper_process_t:
+ * igt_helper_process:
  * @running: indicates whether the process is currently running
- * @use_SIGKILL: whether the helper should be terminated with SIGKILL or SIGQUIT
+ * @use_SIGKILL: whether the helper should be terminated with SIGKILL or SIGTERM
  * @pid: pid of the helper if @running is true
  * @id: internal id
  *
@@ -375,7 +523,7 @@ bool __igt_fork_helper(struct igt_helper_process *proc);
  */
 #define igt_fork_helper(proc) \
 	for (; __igt_fork_helper(proc); exit(0))
-void igt_wait_helper(struct igt_helper_process *proc);
+int igt_wait_helper(struct igt_helper_process *proc);
 void igt_stop_helper(struct igt_helper_process *proc);
 
 /* exit handler code */
@@ -409,41 +557,53 @@ bool igt_run_in_simulation(void);
 
 void igt_skip_on_simulation(void);
 
+extern const char *igt_interactive_debug;
+
 /* structured logging */
 enum igt_log_level {
 	IGT_LOG_DEBUG,
 	IGT_LOG_INFO,
 	IGT_LOG_WARN,
+	IGT_LOG_CRITICAL,
 	IGT_LOG_NONE,
 };
-__attribute__((format(printf, 2, 3)))
-void igt_log(enum igt_log_level level, const char *format, ...);
-__attribute__((format(printf, 2, 0)))
-void igt_vlog(enum igt_log_level level, const char *format, va_list args);
+__attribute__((format(printf, 3, 4)))
+void igt_log(const char *domain, enum igt_log_level level, const char *format, ...);
+__attribute__((format(printf, 3, 0)))
+void igt_vlog(const char *domain, enum igt_log_level level, const char *format, va_list args);
 
 /**
  * igt_debug:
  * @...: format string and optional arguments
  *
- * Wrapper for igt_log for message at the IGT_LOG_DEBUG level.
+ * Wrapper for igt_log() for message at the IGT_LOG_DEBUG level.
  */
-#define igt_debug(f...) igt_log(IGT_LOG_DEBUG, f)
+#define igt_debug(f...) igt_log(IGT_LOG_DOMAIN, IGT_LOG_DEBUG, f)
 
 /**
  * igt_info:
  * @...: format string and optional arguments
  *
- * Wrapper for igt_log for message at the IGT_LOG_INFO level.
+ * Wrapper for igt_log() for message at the IGT_LOG_INFO level.
  */
-#define igt_info(f...) igt_log(IGT_LOG_INFO, f)
+#define igt_info(f...) igt_log(IGT_LOG_DOMAIN, IGT_LOG_INFO, f)
 
 /**
  * igt_warn:
  * @...: format string and optional arguments
  *
- * Wrapper for igt_log for message at the IGT_LOG_WARN level.
+ * Wrapper for igt_log() for message at the IGT_LOG_WARN level.
  */
-#define igt_warn(f...) igt_log(IGT_LOG_WARN, f)
+#define igt_warn(f...) igt_log(IGT_LOG_DOMAIN, IGT_LOG_WARN, f)
+
+/**
+ * igt_critical:
+ * @...: format string and optional arguments
+ *
+ * Wrapper for igt_log() for message at the IGT_LOG_CRITICAL level.
+ */
+#define igt_critical(f...) igt_log(IGT_LOG_DOMAIN, IGT_LOG_CRITICAL, f)
+
 extern enum igt_log_level igt_log_level;
 
 /**
