@@ -3,6 +3,8 @@
 
 #include <stdint.h>
 #include <intel_bufmgr.h>
+#include <i915_drm.h>
+
 #include "igt_core.h"
 #include "intel_reg.h"
 
@@ -14,6 +16,7 @@ struct intel_batchbuffer {
 	uint32_t devid;
 	int gen;
 
+	drm_intel_context *ctx;
 	drm_intel_bo *bo;
 
 	uint8_t buffer[BATCH_SZ];
@@ -24,6 +27,10 @@ struct intel_batchbuffer {
 struct intel_batchbuffer *intel_batchbuffer_alloc(drm_intel_bufmgr *bufmgr,
 						  uint32_t devid);
 
+void intel_batchbuffer_set_context(struct intel_batchbuffer *batch,
+				   drm_intel_context *ctx);
+
+
 void intel_batchbuffer_free(struct intel_batchbuffer *batch);
 
 
@@ -180,6 +187,18 @@ void intel_copy_bo(struct intel_batchbuffer *batch,
 		   drm_intel_bo *dst_bo, drm_intel_bo *src_bo,
 		   long int size);
 
+/*
+ * Yf/Ys tiling
+ *
+ * Tiling mode in the I915_TILING_... namespace for new tiling modes which are
+ * defined in the kernel. (They are not fenceable so the kernel does not need
+ * to know about them.)
+ *
+ * They are to be used the the blitting routines below.
+ */
+#define I915_TILING_Yf	3
+#define I915_TILING_Ys	4
+
 /**
  * igt_buf:
  * @bo: underlying libdrm buffer object
@@ -190,7 +209,7 @@ void intel_copy_bo(struct intel_batchbuffer *batch,
  *
  * This is a i-g-t buffer object wrapper structure which augments the baseline
  * libdrm buffer object with suitable data needed by the render copy and the
- * media fill functions.
+ * fill functions.
  */
 struct igt_buf {
     drm_intel_bo *bo;
@@ -205,6 +224,27 @@ struct igt_buf {
 unsigned igt_buf_width(struct igt_buf *buf);
 unsigned igt_buf_height(struct igt_buf *buf);
 
+void igt_blitter_fast_copy(struct intel_batchbuffer *batch,
+			  struct igt_buf *src, unsigned src_x, unsigned src_y,
+			  unsigned width, unsigned height,
+			  struct igt_buf *dst, unsigned dst_x, unsigned dst_y);
+
+void igt_blitter_fast_copy__raw(int fd,
+				/* src */
+				uint32_t src_handle,
+				unsigned int src_stride,
+				unsigned int src_tiling,
+				unsigned int src_x, unsigned src_y,
+
+				/* size */
+				unsigned int width, unsigned int height,
+
+				/* dst */
+				uint32_t dst_handle,
+				unsigned int dst_stride,
+				unsigned int dst_tiling,
+				unsigned int dst_x, unsigned dst_y);
+
 /**
  * igt_render_copyfunc_t:
  * @batch: batchbuffer object
@@ -235,7 +275,7 @@ typedef void (*igt_render_copyfunc_t)(struct intel_batchbuffer *batch,
 igt_render_copyfunc_t igt_get_render_copyfunc(int devid);
 
 /**
- * igt_media_fillfunc_t:
+ * igt_fillfunc_t:
  * @batch: batchbuffer object
  * @dst: destination i-g-t buffer object
  * @x: destination pixel x-coordination
@@ -244,19 +284,20 @@ igt_render_copyfunc_t igt_get_render_copyfunc(int devid);
  * @height: height of the filled rectangle
  * @color: fill color to use
  *
- * This is the type of the per-platform media fill functions. The
- * platform-specific implementation can be obtained by calling
- * igt_get_media_fillfunc().
+ * This is the type of the per-platform fill functions using media
+ * or gpgpu pipeline. The platform-specific implementation can be obtained
+ * by calling igt_get_media_fillfunc() or igt_get_gpgpu_fillfunc().
  *
- * A media fill function will emit a batchbuffer to the kernel which executes
- * the specified blit fill operation using the media engine.
+ * A fill function will emit a batchbuffer to the kernel which executes
+ * the specified blit fill operation using the media/gpgpu engine.
  */
-typedef void (*igt_media_fillfunc_t)(struct intel_batchbuffer *batch,
-				     struct igt_buf *dst,
-				     unsigned x, unsigned y,
-				     unsigned width, unsigned height,
-				     uint8_t color);
+typedef void (*igt_fillfunc_t)(struct intel_batchbuffer *batch,
+			       struct igt_buf *dst,
+			       unsigned x, unsigned y,
+			       unsigned width, unsigned height,
+			       uint8_t color);
 
-igt_media_fillfunc_t igt_get_media_fillfunc(int devid);
+igt_fillfunc_t igt_get_media_fillfunc(int devid);
+igt_fillfunc_t igt_get_gpgpu_fillfunc(int devid);
 
 #endif
