@@ -54,11 +54,8 @@ struct igt_eviction_test_ops
 static void exchange_uint32_t(void *array, unsigned i, unsigned j)
 {
 	uint32_t *i_arr = array;
-	uint32_t i_tmp;
 
-	i_tmp = i_arr[i];
-	i_arr[i] = i_arr[j];
-	i_arr[j] = i_tmp;
+	igt_swap(i_arr[i], i_arr[j]);
 }
 
 static int minor_evictions(int fd, struct igt_eviction_test_ops *ops,
@@ -77,7 +74,7 @@ static int minor_evictions(int fd, struct igt_eviction_test_ops *ops,
 
 	total_surfaces = gem_aperture_size(fd) / surface_size + 1;
 	igt_require(nr_surfaces < total_surfaces);
-	igt_require(intel_check_memory(total_surfaces, surface_size, CHECK_RAM));
+	intel_require_memory(total_surfaces, surface_size, CHECK_RAM);
 
 	bo = malloc((nr_surfaces + total_surfaces)*sizeof(*bo));
 	igt_assert(bo);
@@ -92,7 +89,7 @@ static int minor_evictions(int fd, struct igt_eviction_test_ops *ops,
 			for (n = 0; n < nr_surfaces; n++, m += 7)
 				sel[n] = bo[m%total_surfaces];
 			ret = ops->copy(fd, sel[0], sel[1], sel, nr_surfaces);
-			igt_assert(ret == 0);
+			igt_assert_eq(ret, 0);
 		}
 		ret = ops->copy(fd, bo[0], bo[0], bo, total_surfaces);
 		igt_assert(ret == ENOSPC);
@@ -112,7 +109,7 @@ static int major_evictions(int fd, struct igt_eviction_test_ops *ops,
 	uint32_t *bo;
 	int ret;
 
-	igt_require(intel_check_memory(nr_surfaces, surface_size, CHECK_RAM));
+	intel_require_memory(nr_surfaces, surface_size, CHECK_RAM);
 
 	bo = malloc(nr_surfaces*sizeof(*bo));
 	igt_assert(bo);
@@ -123,7 +120,7 @@ static int major_evictions(int fd, struct igt_eviction_test_ops *ops,
 	for (loop = 0, m = 0; loop < 100; loop++, m += 17) {
 		n = m % nr_surfaces;
 		ret = ops->copy(fd, bo[n], bo[n], &bo[n], 1);
-		igt_assert(ret == 0);
+		igt_assert_eq(ret, 0);
 	}
 
 	for (n = 0; n < nr_surfaces; n++)
@@ -141,12 +138,12 @@ static int swapping_evictions(int fd, struct igt_eviction_test_ops *ops,
 	uint32_t *bo;
 	int i, n, pass, ret;
 
-	igt_require(intel_check_memory(working_surfaces, surface_size, CHECK_RAM));
+	intel_require_memory(working_surfaces, surface_size, CHECK_RAM);
 
 	if (trash_surfaces < working_surfaces)
 		trash_surfaces = working_surfaces;
 
-	igt_require(intel_check_memory(trash_surfaces, surface_size, CHECK_RAM | CHECK_SWAP));
+	intel_require_memory(trash_surfaces, surface_size, CHECK_RAM | CHECK_SWAP);
 
 	bo = malloc(trash_surfaces*sizeof(*bo));
 	igt_assert(bo);
@@ -159,7 +156,7 @@ static int swapping_evictions(int fd, struct igt_eviction_test_ops *ops,
 
 		for (pass = 0; pass < 100; pass++) {
 			ret = ops->copy(fd, bo[0], bo[1], bo, working_surfaces);
-			igt_assert(ret == 0);
+			igt_assert_eq(ret, 0);
 		}
 	}
 
@@ -170,8 +167,6 @@ static int swapping_evictions(int fd, struct igt_eviction_test_ops *ops,
 	return 0;
 }
 
-#define min(a, b) ((a) < (b) ? (a) : (b))
-
 static int forking_evictions(int fd, struct igt_eviction_test_ops *ops,
 				int surface_size, int working_surfaces,
 				int trash_surfaces, unsigned flags)
@@ -181,7 +176,7 @@ static int forking_evictions(int fd, struct igt_eviction_test_ops *ops,
 	int num_threads = sysconf(_SC_NPROCESSORS_ONLN);
 	int bo_count;
 
-	igt_require(intel_check_memory(working_surfaces, surface_size, CHECK_RAM));
+	intel_require_memory(working_surfaces, surface_size, CHECK_RAM);
 
 	if (flags & FORKING_EVICTIONS_SWAPPING) {
 		bo_count = trash_surfaces;
@@ -191,8 +186,8 @@ static int forking_evictions(int fd, struct igt_eviction_test_ops *ops,
 	} else
 		bo_count = working_surfaces;
 
-	igt_assert(working_surfaces <= bo_count);
-	igt_require(intel_check_memory(bo_count, surface_size, CHECK_RAM | CHECK_SWAP));
+	igt_assert_lte(working_surfaces, bo_count);
+	intel_require_memory(bo_count, surface_size, CHECK_RAM | CHECK_SWAP);
 
 	bo = malloc(bo_count*sizeof(*bo));
 	igt_assert(bo);
@@ -228,7 +223,7 @@ static int forking_evictions(int fd, struct igt_eviction_test_ops *ops,
 
 		for (pass = 0; pass < num_passes; pass++) {
 			ret = ops->copy(realfd, bo[0], bo[1], bo, working_surfaces);
-			igt_assert(ret == 0);
+			igt_assert_eq(ret, 0);
 
 			for (l = 0; l < working_surfaces &&
 			  (flags & FORKING_EVICTIONS_MEMORY_PRESSURE);
