@@ -49,15 +49,26 @@
 
 #include "ioctl_wrappers.h"
 #include "drmtest.h"
+#include "igt_aux.h"
+#include "igt_core.h"
+#include "igt_gt.h"
 #include "intel_bufmgr.h"
 #include "intel_batchbuffer.h"
 #include "intel_io.h"
 #include "intel_chipset.h"
-#include "igt_aux.h"
+
+IGT_TEST_DESCRIPTION("Test of pread/pwrite behavior when writing to active"
+		     " buffers.");
 
 int fd, devid, gen;
 struct intel_batchbuffer *batch;
 
+static void
+nop_release_bo(drm_intel_bo *bo)
+{
+	drm_intel_bo_unreference(bo);
+}
+
 static void
 prw_set_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 {
@@ -104,11 +115,10 @@ unmapped_create_bo(drm_intel_bufmgr *bufmgr, int width, int height)
 static void
 gtt_set_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 {
+	uint32_t *vaddr = bo->virtual;
 	int size = width * height;
-	uint32_t *vaddr;
 
 	drm_intel_gem_bo_start_gtt_access(bo, true);
-	vaddr = bo->virtual;
 	while (size--)
 		*vaddr++ = val;
 }
@@ -116,13 +126,13 @@ gtt_set_bo(drm_intel_bo *bo, uint32_t val, int width, int height)
 static void
 gtt_cmp_bo(drm_intel_bo *bo, uint32_t val, int width, int height, drm_intel_bo *tmp)
 {
-	int size = width * height;
-	uint32_t *vaddr;
+	uint32_t *vaddr = bo->virtual;
+	int y;
 
+	/* GTT access is slow. So we just compare a few points */
 	drm_intel_gem_bo_start_gtt_access(bo, false);
-	vaddr = bo->virtual;
-	while (size--)
-		igt_assert_eq_u32(*vaddr++, val);
+	for (y = 0; y < height; y++)
+		igt_assert_eq_u32(vaddr[y*width+y], val);
 }
 
 static drm_intel_bo *
@@ -159,6 +169,27 @@ gttX_create_bo(drm_intel_bufmgr *bufmgr, int width, int height)
 	return tile_bo(gtt_create_bo(bufmgr, width, height), width);
 }
 
+static drm_intel_bo *
+wc_create_bo(drm_intel_bufmgr *bufmgr, int width, int height)
+{
+	drm_intel_bo *bo;
+
+	gem_require_mmap_wc(fd);
+
+	bo = unmapped_create_bo(bufmgr, width, height);
+	bo->virtual = gem_mmap__wc(fd, bo->handle, 0, bo->size, PROT_READ | PROT_WRITE);
+	return bo;
+}
+
+static void
+wc_release_bo(drm_intel_bo *bo)
+{
+	munmap(bo->virtual, bo->size);
+	bo->virtual = NULL;
+
+	nop_release_bo(bo);
+}
+
 static drm_intel_bo *
 gpu_create_bo(drm_intel_bufmgr *bufmgr, int width, int height)
 {
@@ -272,26 +303,62 @@ gpu_cmp_bo(drm_intel_bo *bo, uint32_t val, int width, int height, drm_intel_bo *
 	cpu_cmp_bo(tmp, val, width, height, NULL);
 }
 
-struct access_mode {
+const struct access_mode {
+	const char *name;
 	void (*set_bo)(drm_intel_bo *bo, uint32_t val, int w, int h);
 	void (*cmp_bo)(drm_intel_bo *bo, uint32_t val, int w, int h, drm_intel_bo *tmp);
 	drm_intel_bo *(*create_bo)(drm_intel_bufmgr *bufmgr, int width, int height);
-	const char *name;
-};
-
-struct access_mode access_modes[] = {
-	{ .set_bo = prw_set_bo, .cmp_bo = prw_cmp_bo,
-		.create_bo = unmapped_create_bo, .name = "prw" },
-	{ .set_bo = cpu_set_bo, .cmp_bo = cpu_cmp_bo,
-		.create_bo = unmapped_create_bo, .name = "cpu" },
-	{ .set_bo = gtt_set_bo, .cmp_bo = gtt_cmp_bo,
-		.create_bo = gtt_create_bo, .name = "gtt" },
-	{ .set_bo = gtt_set_bo, .cmp_bo = gtt_cmp_bo,
-		.create_bo = gttX_create_bo, .name = "gttX" },
-	{ .set_bo = gpu_set_bo, .cmp_bo = gpu_cmp_bo,
-		.create_bo = gpu_create_bo, .name = "gpu" },
-	{ .set_bo = gpu_set_bo, .cmp_bo = gpu_cmp_bo,
-		.create_bo = gpuX_create_bo, .name = "gpuX" },
+	void (*release_bo)(drm_intel_bo *bo);
+} access_modes[] = {
+	{
+		.name = "prw",
+		.set_bo = prw_set_bo,
+		.cmp_bo = prw_cmp_bo,
+		.create_bo = unmapped_create_bo,
+		.release_bo = nop_release_bo,
+	},
+	{
+		.name = "cpu",
+		.set_bo = cpu_set_bo,
+		.cmp_bo = cpu_cmp_bo,
+		.create_bo = unmapped_create_bo,
+		.release_bo = nop_release_bo,
+	},
+	{
+		.name = "gtt",
+		.set_bo = gtt_set_bo,
+		.cmp_bo = gtt_cmp_bo,
+		.create_bo = gtt_create_bo,
+		.release_bo = nop_release_bo,
+	},
+	{
+		.name = "gttX",
+		.set_bo = gtt_set_bo,
+		.cmp_bo = gtt_cmp_bo,
+		.create_bo = gttX_create_bo,
+		.release_bo = nop_release_bo,
+	},
+	{
+		.name = "wc",
+		.set_bo = gtt_set_bo,
+		.cmp_bo = gtt_cmp_bo,
+		.create_bo = wc_create_bo,
+		.release_bo = wc_release_bo,
+	},
+	{
+		.name = "gpu",
+		.set_bo = gpu_set_bo,
+		.cmp_bo = gpu_cmp_bo,
+		.create_bo = gpu_create_bo,
+		.release_bo = nop_release_bo,
+	},
+	{
+		.name = "gpuX",
+		.set_bo = gpu_set_bo,
+		.cmp_bo = gpu_cmp_bo,
+		.create_bo = gpuX_create_bo,
+		.release_bo = nop_release_bo,
+	},
 };
 
 #define MAX_NUM_BUFFERS 1024
@@ -300,6 +367,7 @@ const int width = 512, height = 512;
 igt_render_copyfunc_t rendercopy;
 
 typedef void (*do_copy)(drm_intel_bo *dst, drm_intel_bo *src);
+typedef struct igt_hang_ring (*do_hang)(void);
 
 static void render_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
 {
@@ -333,11 +401,89 @@ static void blt_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
 		       width, height, 32);
 }
 
-static void do_overwrite_source(struct access_mode *mode,
+static void cpu_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
+{
+	const int size = width * height * sizeof(uint32_t);
+	void *d, *s;
+
+	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_CPU, 0);
+	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
+	s = gem_mmap__cpu(fd, src->handle, 0, size, PROT_READ);
+	igt_assert(s != NULL);
+	d = gem_mmap__cpu(fd, dst->handle, 0, size, PROT_WRITE);
+	igt_assert(d != NULL);
+
+	memcpy(d, s, size);
+
+	munmap(d, size);
+	munmap(s, size);
+}
+
+static void gtt_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
+{
+	const int size = width * height * sizeof(uint32_t);
+	void *d, *s;
+
+	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_GTT, 0);
+	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+
+	s = gem_mmap__gtt(fd, src->handle, size, PROT_READ);
+	igt_assert(s != NULL);
+	d = gem_mmap__gtt(fd, dst->handle, size, PROT_WRITE);
+	igt_assert(d != NULL);
+
+	memcpy(d, s, size);
+
+	munmap(d, size);
+	munmap(s, size);
+}
+
+static void wc_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
+{
+	const int size = width * height * sizeof(uint32_t);
+	void *d, *s;
+
+	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_GTT, 0);
+	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+
+	s = gem_mmap__wc(fd, src->handle, 0, size, PROT_READ);
+	igt_assert(s != NULL);
+	d = gem_mmap__wc(fd, dst->handle, 0, size, PROT_WRITE);
+	igt_assert(d != NULL);
+
+	memcpy(d, s, size);
+
+	munmap(d, size);
+	munmap(s, size);
+}
+
+static struct igt_hang_ring no_hang(void)
+{
+	return (struct igt_hang_ring){0, 0};
+}
+
+static struct igt_hang_ring bcs_hang(void)
+{
+	return igt_hang_ring(fd, I915_EXEC_BLT);
+}
+
+static struct igt_hang_ring rcs_hang(void)
+{
+	return igt_hang_ring(fd, I915_EXEC_RENDER);
+}
+
+static void hang_require(void)
+{
+	igt_require_hang_ring(fd, -1);
+}
+
+static void do_overwrite_source(const struct access_mode *mode,
 				drm_intel_bo **src, drm_intel_bo **dst,
 				drm_intel_bo *dummy,
-				do_copy do_copy_func)
+				do_copy do_copy_func,
+				do_hang do_hang_func)
 {
+	struct igt_hang_ring hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -347,17 +493,63 @@ static void do_overwrite_source(struct access_mode *mode,
 	}
 	for (i = 0; i < num_buffers; i++)
 		do_copy_func(dst[i], src[i]);
+	hang = do_hang_func();
 	for (i = num_buffers; i--; )
 		mode->set_bo(src[i], 0xdeadbeef, width, height);
 	for (i = 0; i < num_buffers; i++)
 		mode->cmp_bo(dst[i], i, width, height, dummy);
+	igt_post_hang_ring(fd, hang);
 }
 
-static void do_early_read(struct access_mode *mode,
+static void do_overwrite_source__rev(const struct access_mode *mode,
+				     drm_intel_bo **src, drm_intel_bo **dst,
+				     drm_intel_bo *dummy,
+				     do_copy do_copy_func,
+				     do_hang do_hang_func)
+{
+	struct igt_hang_ring hang;
+	int i;
+
+	gem_quiescent_gpu(fd);
+	for (i = 0; i < num_buffers; i++) {
+		mode->set_bo(src[i], i, width, height);
+		mode->set_bo(dst[i], ~i, width, height);
+	}
+	for (i = 0; i < num_buffers; i++)
+		do_copy_func(dst[i], src[i]);
+	hang = do_hang_func();
+	for (i = 0; i < num_buffers; i++)
+		mode->set_bo(src[i], 0xdeadbeef, width, height);
+	for (i = num_buffers; i--; )
+		mode->cmp_bo(dst[i], i, width, height, dummy);
+	igt_post_hang_ring(fd, hang);
+}
+
+static void do_overwrite_source__one(const struct access_mode *mode,
+				     drm_intel_bo **src, drm_intel_bo **dst,
+				     drm_intel_bo *dummy,
+				     do_copy do_copy_func,
+				     do_hang do_hang_func)
+{
+	struct igt_hang_ring hang;
+
+	gem_quiescent_gpu(fd);
+	mode->set_bo(src[0], 0, width, height);
+	mode->set_bo(dst[0], ~0, width, height);
+	do_copy_func(dst[0], src[0]);
+	hang = do_hang_func();
+	mode->set_bo(src[0], 0xdeadbeef, width, height);
+	mode->cmp_bo(dst[0], 0, width, height, dummy);
+	igt_post_hang_ring(fd, hang);
+}
+
+static void do_early_read(const struct access_mode *mode,
 			  drm_intel_bo **src, drm_intel_bo **dst,
 			  drm_intel_bo *dummy,
-			  do_copy do_copy_func)
+			  do_copy do_copy_func,
+			  do_hang do_hang_func)
 {
+	struct igt_hang_ring hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -365,15 +557,19 @@ static void do_early_read(struct access_mode *mode,
 		mode->set_bo(src[i], 0xdeadbeef, width, height);
 	for (i = 0; i < num_buffers; i++)
 		do_copy_func(dst[i], src[i]);
+	hang = do_hang_func();
 	for (i = num_buffers; i--; )
 		mode->cmp_bo(dst[i], 0xdeadbeef, width, height, dummy);
+	igt_post_hang_ring(fd, hang);
 }
 
-static void do_gpu_read_after_write(struct access_mode *mode,
+static void do_gpu_read_after_write(const struct access_mode *mode,
 				    drm_intel_bo **src, drm_intel_bo **dst,
 				    drm_intel_bo *dummy,
-				    do_copy do_copy_func)
+				    do_copy do_copy_func,
+				    do_hang do_hang_func)
 {
+	struct igt_hang_ring hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -383,74 +579,85 @@ static void do_gpu_read_after_write(struct access_mode *mode,
 		do_copy_func(dst[i], src[i]);
 	for (i = num_buffers; i--; )
 		do_copy_func(dummy, dst[i]);
+	hang = do_hang_func();
 	for (i = num_buffers; i--; )
 		mode->cmp_bo(dst[i], 0xabcdabcd, width, height, dummy);
+	igt_post_hang_ring(fd, hang);
 }
 
-typedef void (*do_test)(struct access_mode *mode,
+typedef void (*do_test)(const struct access_mode *mode,
 			drm_intel_bo **src, drm_intel_bo **dst,
 			drm_intel_bo *dummy,
-			do_copy do_copy_func);
+			do_copy do_copy_func,
+			do_hang do_hang_func);
 
-typedef void (*run_wrap)(struct access_mode *mode,
+typedef void (*run_wrap)(const struct access_mode *mode,
 			 drm_intel_bo **src, drm_intel_bo **dst,
 			 drm_intel_bo *dummy,
 			 do_test do_test_func,
-			 do_copy do_copy_func);
+			 do_copy do_copy_func,
+			 do_hang do_hang_func);
 
-static void run_single(struct access_mode *mode,
+static void run_single(const struct access_mode *mode,
 		       drm_intel_bo **src, drm_intel_bo **dst,
 		       drm_intel_bo *dummy,
 		       do_test do_test_func,
-		       do_copy do_copy_func)
+		       do_copy do_copy_func,
+		       do_hang do_hang_func)
 {
-	do_test_func(mode, src, dst, dummy, do_copy_func);
+	do_test_func(mode, src, dst, dummy, do_copy_func, do_hang_func);
 }
 
-static void run_interruptible(struct access_mode *mode,
+static void run_interruptible(const struct access_mode *mode,
 			      drm_intel_bo **src, drm_intel_bo **dst,
 			      drm_intel_bo *dummy,
 			      do_test do_test_func,
-			      do_copy do_copy_func)
+			      do_copy do_copy_func,
+			      do_hang do_hang_func)
 {
 	int loop;
 
 	for (loop = 0; loop < 10; loop++)
-		do_test_func(mode, src, dst, dummy, do_copy_func);
+		do_test_func(mode, src, dst, dummy, do_copy_func, do_hang_func);
 }
 
-static void run_forked(struct access_mode *mode,
+static void run_forked(const struct access_mode *mode,
 		       drm_intel_bo **src, drm_intel_bo **dst,
 		       drm_intel_bo *dummy,
 		       do_test do_test_func,
-		       do_copy do_copy_func)
+		       do_copy do_copy_func,
+		       do_hang do_hang_func)
 {
 	const int old_num_buffers = num_buffers;
-	drm_intel_bufmgr *bufmgr;
 
 	num_buffers /= 16;
 	num_buffers += 2;
 
 	igt_fork(child, 16) {
+		drm_intel_bufmgr *bufmgr;
+
 		/* recreate process local variables */
+		fd = drm_open_any();
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		drm_intel_bufmgr_gem_enable_reuse(bufmgr);
+
 		batch = intel_batchbuffer_alloc(bufmgr, devid);
+
 		for (int i = 0; i < num_buffers; i++) {
 			src[i] = mode->create_bo(bufmgr, width, height);
 			dst[i] = mode->create_bo(bufmgr, width, height);
 		}
 		dummy = mode->create_bo(bufmgr, width, height);
+
 		for (int loop = 0; loop < 10; loop++)
-			do_test_func(mode, src, dst, dummy, do_copy_func);
-		/* as we borrow the fd, we need to reap our bo */
+			do_test_func(mode, src, dst, dummy,
+				     do_copy_func, do_hang_func);
+
 		for (int i = 0; i < num_buffers; i++) {
-			drm_intel_bo_unreference(src[i]);
-			drm_intel_bo_unreference(dst[i]);
+			mode->release_bo(src[i]);
+			mode->release_bo(dst[i]);
 		}
-		drm_intel_bo_unreference(dummy);
-		intel_batchbuffer_free(batch);
-		drm_intel_bufmgr_destroy(bufmgr);
+		mode->release_bo(dummy);
 	}
 
 	igt_waitchildren();
@@ -458,6 +665,40 @@ static void run_forked(struct access_mode *mode,
 	num_buffers = old_num_buffers;
 }
 
+static void bit17_require(void)
+{
+	struct drm_i915_gem_get_tiling2 {
+		uint32_t handle;
+		uint32_t tiling_mode;
+		uint32_t swizzle_mode;
+		uint32_t phys_swizzle_mode;
+	} arg;
+#define DRM_IOCTL_I915_GEM_GET_TILING2	DRM_IOWR (DRM_COMMAND_BASE + DRM_I915_GEM_GET_TILING, struct drm_i915_gem_get_tiling2)
+
+	memset(&arg, 0, sizeof(arg));
+	arg.handle = gem_create(fd, 4096);
+	gem_set_tiling(fd, arg.handle, I915_TILING_X, 512);
+
+	do_or_die(drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_TILING2, &arg));
+	gem_close(fd, arg.handle);
+	igt_require(arg.phys_swizzle_mode == arg.swizzle_mode);
+}
+
+static void cpu_require(void)
+{
+	bit17_require();
+}
+
+static void gtt_require(void)
+{
+}
+
+static void wc_require(void)
+{
+	bit17_require();
+	gem_require_mmap_wc(fd);
+}
+
 static void bcs_require(void)
 {
 }
@@ -467,82 +708,121 @@ static void rcs_require(void)
 	igt_require(rendercopy);
 }
 
+static void no_require(void)
+{
+}
+
 static void
-run_basic_modes(struct access_mode *mode,
-		drm_intel_bo **src, drm_intel_bo **dst,
-		drm_intel_bo *dummy, const char *suffix,
+run_basic_modes(const struct access_mode *mode,
+		const char *suffix,
 		run_wrap run_wrap_func)
 {
-	struct {
+	const struct {
 		const char *prefix;
 		do_copy copy;
 		void (*require)(void);
 	} pipelines[] = {
-		{ "bcs", blt_copy_bo, bcs_require },
-		{ "rcs", render_copy_bo, rcs_require },
+		{ "cpu", cpu_copy_bo, cpu_require },
+		{ "gtt", gtt_copy_bo, gtt_require },
+		{ "wc", wc_copy_bo, wc_require },
+		{ "blt", blt_copy_bo, bcs_require },
+		{ "render", render_copy_bo, rcs_require },
 		{ NULL, NULL }
 	}, *p;
+	const struct {
+		const char *suffix;
+		do_hang hang;
+		void (*require)(void);
+	} hangs[] = {
+		{ "", no_hang, no_require },
+		{ "-hang-blt", bcs_hang, hang_require },
+		{ "-hang-render", rcs_hang, hang_require },
+		{ NULL, NULL },
+	}, *h;
+	drm_intel_bo *src[MAX_NUM_BUFFERS], *dst[MAX_NUM_BUFFERS], *dummy = NULL;
+	drm_intel_bufmgr *bufmgr;
 
-	for (p = pipelines; p->prefix; p++) {
-		/* try to overwrite the source values */
-		igt_subtest_f("%s-%s-overwrite-source%s", mode->name, p->prefix, suffix) {
-			p->require();
-			run_wrap_func(mode, src, dst, dummy,
-				      do_overwrite_source, p->copy);
-		}
 
-		/* try to read the results before the copy completes */
-		igt_subtest_f("%s-%s-early-read%s", mode->name, p->prefix, suffix) {
-			p->require();
-			run_wrap_func(mode, src, dst, dummy,
-				      do_early_read, p->copy);
-		}
+	for (h = hangs; h->suffix; h++) {
+		for (p = pipelines; p->prefix; p++) {
+			igt_fixture {
+				bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
+				drm_intel_bufmgr_gem_enable_reuse(bufmgr);
+				batch = intel_batchbuffer_alloc(bufmgr, devid);
 
-		/* and finally try to trick the kernel into loosing the pending write */
-		igt_subtest_f("%s-%s-gpu-read-after-write%s", mode->name, p->prefix, suffix) {
-			p->require();
-			run_wrap_func(mode, src, dst, dummy,
-				      do_gpu_read_after_write, p->copy);
+				for (int i = 0; i < num_buffers; i++) {
+					src[i] = mode->create_bo(bufmgr, width, height);
+					dst[i] = mode->create_bo(bufmgr, width, height);
+				}
+				dummy = mode->create_bo(bufmgr, width, height);
+			}
+
+			/* try to overwrite the source values */
+			igt_subtest_f("%s-%s-overwrite-source-one%s%s", mode->name, p->prefix, suffix, h->suffix) {
+				h->require();
+				p->require();
+				run_wrap_func(mode, src, dst, dummy,
+					      do_overwrite_source__one,
+					      p->copy, h->hang);
+			}
+
+			igt_subtest_f("%s-%s-overwrite-source%s%s", mode->name, p->prefix, suffix, h->suffix) {
+				h->require();
+				p->require();
+				run_wrap_func(mode, src, dst, dummy,
+					      do_overwrite_source,
+					      p->copy, h->hang);
+			}
+			igt_subtest_f("%s-%s-overwrite-source-rev%s%s", mode->name, p->prefix, suffix, h->suffix) {
+				h->require();
+				p->require();
+				run_wrap_func(mode, src, dst, dummy,
+					      do_overwrite_source__rev,
+					      p->copy, h->hang);
+			}
+
+			/* try to read the results before the copy completes */
+			igt_subtest_f("%s-%s-early-read%s%s", mode->name, p->prefix, suffix, h->suffix) {
+				h->require();
+				p->require();
+				run_wrap_func(mode, src, dst, dummy,
+					      do_early_read,
+					      p->copy, h->hang);
+			}
+
+			/* and finally try to trick the kernel into loosing the pending write */
+			igt_subtest_f("%s-%s-gpu-read-after-write%s%s", mode->name, p->prefix, suffix, h->suffix) {
+				h->require();
+				p->require();
+				run_wrap_func(mode, src, dst, dummy,
+					      do_gpu_read_after_write,
+					      p->copy, h->hang);
+			}
+
+			igt_fixture {
+				for (int i = 0; i < num_buffers; i++) {
+					mode->release_bo(src[i]);
+					mode->release_bo(dst[i]);
+				}
+				mode->release_bo(dummy);
+				intel_batchbuffer_free(batch);
+				drm_intel_bufmgr_destroy(bufmgr);
+			}
 		}
 	}
 }
 
 static void
-run_modes(struct access_mode *mode)
+run_modes(const struct access_mode *mode)
 {
-	drm_intel_bo *src[MAX_NUM_BUFFERS], *dst[MAX_NUM_BUFFERS], *dummy = NULL;
-	drm_intel_bufmgr *bufmgr;
-
-	igt_fixture {
-		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
-		drm_intel_bufmgr_gem_enable_reuse(bufmgr);
-		batch = intel_batchbuffer_alloc(bufmgr, devid);
-
-		for (int i = 0; i < num_buffers; i++) {
-			src[i] = mode->create_bo(bufmgr, width, height);
-			dst[i] = mode->create_bo(bufmgr, width, height);
-		}
-		dummy = mode->create_bo(bufmgr, width, height);
-	}
-
-	run_basic_modes(mode, src, dst, dummy, "", run_single);
+	run_basic_modes(mode, "", run_single);
 
 	igt_fork_signal_helper();
-	run_basic_modes(mode, src, dst, dummy, "-interruptible", run_interruptible);
+	run_basic_modes(mode, "-interruptible", run_interruptible);
 	igt_stop_signal_helper();
 
-	igt_fixture {
-		for (int i = 0; i < num_buffers; i++) {
-			drm_intel_bo_unreference(src[i]);
-			drm_intel_bo_unreference(dst[i]);
-		}
-		drm_intel_bo_unreference(dummy);
-		intel_batchbuffer_free(batch);
-		drm_intel_bufmgr_destroy(bufmgr);
-	}
-
 	igt_fork_signal_helper();
-	run_basic_modes(mode, src, dst, dummy, "-forked", run_forked);
+	run_basic_modes(mode, "-forked", run_forked);
 	igt_stop_signal_helper();
 }
 
