@@ -31,61 +31,53 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "i915_drm.h"
+#include <errno.h>
+
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 
-struct local_drm_i915_context_destroy {
-	__u32 ctx_id;
-	__u32 pad;
-};
+IGT_TEST_DESCRIPTION("Negative test cases for destroy contexts.");
 
-#define CONTEXT_DESTROY_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x2e, struct local_drm_i915_context_destroy)
+uint32_t ctx_id;
+int fd;
 
-static void handle_bad(int ret, int lerrno, int expected, const char *desc)
+igt_main
 {
-	if (ret != 0 && lerrno != expected) {
-		fprintf(stderr, "%s - errno was %d, but should have been %d\n",
-				desc, lerrno, expected);
-		igt_fail(1);
-	} else if (ret == 0) {
-		fprintf(stderr, "%s - Command succeeded, but should have failed\n",
-			desc);
-		igt_fail(1);
+	igt_fixture {
+		fd = drm_open_any_render();
+
+		ctx_id = gem_context_create(fd);
+		/* Make sure a proper destroy works first */
+		gem_context_destroy(fd, ctx_id);
 	}
-}
-
-int main(int argc, char *argv[])
-{
-	struct local_drm_i915_context_destroy destroy;
-	uint32_t ctx_id;
-	int ret, fd;
-
-	igt_skip_on_simulation();
-
-	fd = drm_open_any_render();
-
-	ctx_id = gem_context_create(fd);
-
-	destroy.ctx_id = ctx_id;
-	/* Make sure a proper destroy works first */
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	igt_assert(ret == 0);
 
 	/* try double destroy */
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	handle_bad(ret, errno, ENOENT, "double destroy");
+	igt_subtest("double-destroy") {
+		ctx_id = gem_context_create(fd);
+		gem_context_destroy(fd, ctx_id);
+		igt_assert(__gem_context_destroy(fd, ctx_id) == -ENOENT);
+	}
 
-	/* destroy something random */
-	destroy.ctx_id = 2;
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	handle_bad(ret, errno, ENOENT, "random destroy");
+	igt_subtest("invalid-ctx")
+		igt_assert(__gem_context_destroy(fd, 2) == -ENOENT);
 
-	/* Try to destroy the default context */
-	destroy.ctx_id = 0;
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	handle_bad(ret, errno, ENOENT, "default destroy");
+	igt_subtest("invalid-default-ctx")
+		igt_assert(__gem_context_destroy(fd, 0) == -ENOENT);
 
-	close(fd);
+	igt_subtest("invalid-pad") {
+		struct drm_i915_gem_context_destroy destroy;
 
-	igt_success();
+		ctx_id = gem_context_create(fd);
+
+		memset(&destroy, 0, sizeof(destroy));
+		destroy.ctx_id = ctx_id;
+		destroy.pad = 1;
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy) < 0 &&
+			   errno == EINVAL);
+		gem_context_destroy(fd, ctx_id);
+	}
+
+	igt_fixture
+		close(fd);
 }
