@@ -46,10 +46,12 @@
 #include "drm.h"
 #include "ioctl_wrappers.h"
 #include "drmtest.h"
+#include "igt_aux.h"
 
-#define BATCH_SIZE		(1024*1024)
+IGT_TEST_DESCRIPTION("Run a large nop batch to stress test the error capture"
+		     " code.");
 
-static void exec(int fd, uint32_t handle, uint32_t reloc_ofs)
+static void exec1(int fd, uint32_t handle, uint32_t reloc_ofs, unsigned flags, uint32_t *ptr)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 gem_exec[1];
@@ -80,7 +82,7 @@ static void exec(int fd, uint32_t handle, uint32_t reloc_ofs)
 	execbuf.num_cliprects = 0;
 	execbuf.DR1 = 0;
 	execbuf.DR4 = 0;
-	execbuf.flags = 0;
+	execbuf.flags = flags;
 	i915_execbuffer2_set_context_id(execbuf, 0);
 	execbuf.rsvd2 = 0;
 
@@ -93,34 +95,113 @@ static void exec(int fd, uint32_t handle, uint32_t reloc_ofs)
 
 	igt_warn_on(gem_reloc[0].presumed_offset == -1);
 
-	gem_read(fd, handle, reloc_ofs, &tmp, 4);
-	igt_assert_eq(tmp, gem_reloc[0].presumed_offset);
+	if (ptr) {
+		igt_assert_eq(ptr[reloc_ofs/sizeof(*ptr)],
+			      gem_reloc[0].presumed_offset);
+	} else {
+		gem_read(fd, handle, reloc_ofs, &tmp, 4);
+		igt_assert_eq(tmp, gem_reloc[0].presumed_offset);
+	}
+}
+
+static void execN(int fd, uint32_t handle, uint32_t batch_size, unsigned flags, uint32_t *ptr)
+{
+#define reloc_ofs(N, T) ((((N)+1) << 12) - 4*(1 + ((N) == ((T)-1))))
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 gem_exec[1];
+	struct drm_i915_gem_relocation_entry *gem_reloc;
+	int n, nreloc = batch_size >> 12;
+	uint32_t tmp;
+
+	gem_reloc = calloc(nreloc, sizeof(*gem_reloc));
+	igt_assert(gem_reloc);
+
+	for (n = 0; n < nreloc; n++) {
+		gem_reloc[n].offset = reloc_ofs(n, nreloc);
+		gem_reloc[n].target_handle = handle;
+		gem_reloc[n].read_domains = I915_GEM_DOMAIN_RENDER;
+	}
+
+	memset(gem_exec, 0, sizeof(gem_exec));
+	gem_exec[0].handle = handle;
+	gem_exec[0].relocation_count = nreloc;
+	gem_exec[0].relocs_ptr = (uintptr_t)gem_reloc;
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffer_count = 1;
+	execbuf.batch_start_offset = 0;
+	execbuf.batch_len = 8;
+	execbuf.flags = flags;
+
+	/* Avoid hitting slowpaths in the reloc processing which might yield a
+	 * presumed_offset of -1. Happens when the batch is still busy from the
+	 * last round. */
+	gem_sync(fd, handle);
+
+	gem_execbuf(fd, &execbuf);
+	for (n = 0; n < nreloc; n++)
+		igt_warn_on(gem_reloc[n].presumed_offset == -1);
+
+	if (ptr) {
+		for (n = 0; n < nreloc; n++)
+			igt_assert_eq(ptr[reloc_ofs(n, nreloc)/sizeof(*ptr)],
+				      gem_reloc[n].presumed_offset);
+	} else {
+		for (n = 0; n < nreloc; n++) {
+			gem_read(fd, handle, reloc_ofs(n, nreloc), &tmp, 4);
+			igt_assert_eq(tmp, gem_reloc[n].presumed_offset);
+		}
+	}
+
+	free(gem_reloc);
+#undef reloc_ofs
 }
 
 igt_simple_main
 {
 	uint32_t batch[2] = {MI_BATCH_BUFFER_END};
-	uint32_t handle;
 	int fd;
 	uint32_t reloc_ofs;
 	unsigned batch_size;
-
-	igt_skip_on_simulation();
+	int max;
 
 	fd = drm_open_any();
+	max = 3 * gem_aperture_size(fd) / 4;
+
+	intel_require_memory(1, max, CHECK_RAM);
+
+	for (batch_size = 4096; batch_size <= max; ) {
+		uint32_t handle, *ptr;
 
-	for (batch_size = BATCH_SIZE/4; batch_size <= BATCH_SIZE; batch_size += 4096) {
 		handle = gem_create(fd, batch_size);
 		gem_write(fd, handle, 0, batch, sizeof(batch));
 
+		ptr = NULL;
+		if (gem_mmap__has_wc(fd))
+			ptr = gem_mmap__wc(fd, handle, 0, batch_size, PROT_READ);
+
 		for (reloc_ofs = 4096; reloc_ofs < batch_size; reloc_ofs += 4096) {
 			igt_debug("batch_size %u, reloc_ofs %u\n",
 				  batch_size, reloc_ofs);
-			exec(fd, handle, reloc_ofs);
+			exec1(fd, handle, reloc_ofs, 0, ptr);
+			exec1(fd, handle, reloc_ofs, I915_EXEC_SECURE, ptr);
 		}
-	}
 
-	gem_close(fd, handle);
+		igt_debug("batch_size %u, all %d relocs\n",
+			  batch_size, batch_size >> 12);
+		execN(fd, handle, batch_size, 0, ptr);
+		execN(fd, handle, batch_size, I915_EXEC_SECURE, ptr);
+
+		if (ptr)
+			munmap(ptr, batch_size);
+		gem_madvise(fd, handle, I915_MADV_DONTNEED);
+
+		if (batch_size < max && 2*batch_size > max)
+			batch_size = max;
+		else
+			batch_size *= 2;
+	}
 
 	close(fd);
 }
