@@ -30,19 +30,18 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
 #include <fcntl.h>
 #include <inttypes.h>
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <sys/mman.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
+#include "intel_io.h"
 
-#define MI_BATCH_BUFFER_END	(0xA<<23)
+#define LOCAL_I915_EXEC_VEBOX (4<<0)
 
 static double elapsed(const struct timeval *start,
 		      const struct timeval *end,
@@ -51,7 +50,7 @@ static double elapsed(const struct timeval *start,
 	return (1e6*(end->tv_sec - start->tv_sec) + (end->tv_usec - start->tv_usec))/loop;
 }
 
-static int exec(int fd, uint32_t handle, int loops)
+static int exec(int fd, uint32_t handle, int loops, unsigned ring_id)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 gem_exec[1];
@@ -74,7 +73,7 @@ static int exec(int fd, uint32_t handle, int loops)
 	execbuf.num_cliprects = 0;
 	execbuf.DR1 = 0;
 	execbuf.DR4 = 0;
-	execbuf.flags = 0;
+	execbuf.flags = ring_id;
 	i915_execbuffer2_set_context_id(execbuf, 0);
 	execbuf.rsvd2 = 0;
 
@@ -88,32 +87,52 @@ static int exec(int fd, uint32_t handle, int loops)
 	return ret;
 }
 
-int main(int argc, char **argv)
+static void loop(int fd, uint32_t handle, unsigned ring_id, const char *ring_name)
 {
-	uint32_t batch[2] = {MI_BATCH_BUFFER_END};
-	uint32_t handle;
 	int count;
-	int fd;
 
-	fd = drm_open_any();
+	gem_require_ring(fd, ring_id);
 
-	handle = gem_create(fd, 4096);
-	gem_write(fd, handle, 0, batch, sizeof(batch));
-
-	for (count = 1; count <= 1<<17; count <<= 1) {
+	for (count = 1; count <= SLOW_QUICK(1<<17, 1<<4); count <<= 1) {
 		struct timeval start, end;
 
 		gettimeofday(&start, NULL);
-		if (exec(fd, handle, count))
-			exit(1);
+		igt_assert_eq(exec(fd, handle, count, ring_id), 0);
 		gettimeofday(&end, NULL);
-		printf("Time to exec x %d:		%7.3fµs\n",
-		       count, elapsed(&start, &end, count));
+		igt_info("Time to exec x %d:		%7.3fµs (ring=%s)\n",
+			 count, elapsed(&start, &end, count), ring_name);
 		fflush(stdout);
 	}
-	gem_close(fd, handle);
+}
 
-	close(fd);
+uint32_t batch[2] = {MI_BATCH_BUFFER_END};
+uint32_t handle;
+int fd;
 
-	return 0;
+igt_main
+{
+	igt_fixture {
+		fd = drm_open_any();
+
+		handle = gem_create(fd, 4096);
+		gem_write(fd, handle, 0, batch, sizeof(batch));
+	}
+
+	igt_subtest("render")
+		loop(fd, handle, I915_EXEC_RENDER, "render");
+
+	igt_subtest("bsd")
+		loop(fd, handle, I915_EXEC_BSD, "bsd");
+
+	igt_subtest("blt")
+		loop(fd, handle, I915_EXEC_BLT, "blt");
+
+	igt_subtest("vebox")
+		loop(fd, handle, LOCAL_I915_EXEC_VEBOX, "vebox");
+
+	igt_fixture {
+		gem_close(fd, handle);
+
+		close(fd);
+	}
 }
