@@ -50,6 +50,9 @@
 #include "intel_io.h"
 #include "igt_aux.h"
 
+IGT_TEST_DESCRIPTION("Test doing many blits with a working set larger than the"
+		     " aperture size.");
+
 #define WIDTH 512
 #define HEIGHT 512
 
@@ -150,7 +153,7 @@ copy(int fd, uint32_t dst, uint32_t src)
 		drmCommandNone(fd, DRM_I915_GEM_THROTTLE);
 		ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
 	}
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	gem_close(fd, handle);
 }
@@ -192,6 +195,8 @@ static void run_test(int fd, int count)
 	uint32_t start = 0;
 	int i;
 
+	igt_debug("Using %d 1MiB buffers\n", count);
+
 	handle = malloc(sizeof(uint32_t)*count*2);
 	start_val = handle + count;
 
@@ -248,36 +253,32 @@ static void run_test(int fd, int count)
 
 int main(int argc, char **argv)
 {
-	int fd = 0, count = 0;
-
-	igt_skip_on_simulation();
+	int fd = 0;
 
 	igt_subtest_init(argc, argv);
 
 	igt_fixture {
 		fd = drm_open_any();
-
-		if (argc > 1)
-			count = atoi(argv[1]);
-		if (count == 0)
-			count = 3 * gem_aperture_size(fd) / (1024*1024) / 2;
-		else if (count < 2) {
-			fprintf(stderr, "count must be >= 2\n");
-			return 1;
-		}
-
-		if (count > intel_get_total_ram_mb() * 9 / 10) {
-			count = intel_get_total_ram_mb() * 9 / 10;
-			igt_info("not enough RAM to run test, reducing buffer count\n");
-		}
-
-		igt_info("Using %d 1MiB buffers\n", count);
 	}
 
-	igt_subtest("normal")
+	igt_subtest("basic")
+		run_test(fd, 2);
+
+	igt_subtest("normal") {
+		int count;
+
+		count = 3 * gem_aperture_size(fd) / (1024*1024) / 2;
+		igt_require(count > 1);
+		intel_require_memory(count, sizeof(linear), CHECK_RAM);
 		run_test(fd, count);
+	}
 
 	igt_subtest("interruptible") {
+		int count;
+
+		count = 3 * gem_aperture_size(fd) / (1024*1024) / 2;
+		igt_require(count > 1);
+		intel_require_memory(count, sizeof(linear), CHECK_RAM);
 		igt_fork_signal_helper();
 		run_test(fd, count);
 		igt_stop_signal_helper();
