@@ -42,7 +42,7 @@
 
 #include "ioctl_wrappers.h"
 #include "drmtest.h"
-#include "igt_debugfs.h"
+#include "igt_gt.h"
 #include "intel_chipset.h"
 #include "intel_io.h"
 #include "igt_aux.h"
@@ -53,7 +53,6 @@
 #define RS_UNKNOWN       (1 << 2)
 
 static uint32_t devid;
-static bool hw_contexts;
 
 struct local_drm_i915_reset_stats {
 	__u32 ctx_id;
@@ -64,20 +63,8 @@ struct local_drm_i915_reset_stats {
 	__u32 pad;
 };
 
-struct local_drm_i915_gem_context_create {
-	__u32 ctx_id;
-	__u32 pad;
-};
-
-struct local_drm_i915_gem_context_destroy {
-	__u32 ctx_id;
-	__u32 pad;
-};
-
 #define MAX_FD 32
 
-#define CONTEXT_CREATE_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x2d, struct local_drm_i915_gem_context_create)
-#define CONTEXT_DESTROY_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x2e, struct local_drm_i915_gem_context_destroy)
 #define GET_RESET_STATS_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x32, struct local_drm_i915_reset_stats)
 
 #define LOCAL_I915_EXEC_VEBOX	(4 << 0)
@@ -89,64 +76,32 @@ static bool gem_has_render(int fd)
 	return true;
 }
 
-static bool has_context(const struct target_ring *ring);
-
 static const struct target_ring {
 	uint32_t exec;
 	bool (*present)(int fd);
-	bool (*contexts)(const struct target_ring *ring);
 	const char *name;
 } rings[] = {
-	{ I915_EXEC_RENDER, gem_has_render, has_context, "render" },
-	{ I915_EXEC_BLT, gem_has_blt, has_context, "blt" },
-	{ I915_EXEC_BSD, gem_has_bsd, has_context, "bsd" },
-	{ LOCAL_I915_EXEC_VEBOX, gem_has_vebox, has_context, "vebox" },
+	{ I915_EXEC_RENDER, gem_has_render, "render" },
+	{ I915_EXEC_BLT, gem_has_blt, "blt" },
+	{ I915_EXEC_BSD, gem_has_bsd, "bsd" },
+	{ LOCAL_I915_EXEC_VEBOX, gem_has_vebox, "vebox" },
 };
 
-static bool has_context(const struct target_ring *ring)
+static void check_context(const struct target_ring *ring)
 {
-	if (!hw_contexts)
-		return false;
+	int fd = drm_open_any();
 
-	if(ring->exec == I915_EXEC_RENDER)
-		return true;
+	gem_context_destroy(fd,
+			    gem_context_create(fd));
+	close(fd);
 
-	return false;
+	igt_require(ring->exec == I915_EXEC_RENDER);
 }
 
 #define NUM_RINGS (sizeof(rings)/sizeof(struct target_ring))
 
 static const struct target_ring *current_ring;
 
-static uint32_t context_create(int fd)
-{
-	struct local_drm_i915_gem_context_create create;
-	int ret;
-
-	create.ctx_id = rand();
-	create.pad = rand();
-
-	ret = drmIoctl(fd, CONTEXT_CREATE_IOCTL, &create);
-	igt_assert(ret == 0);
-
-	return create.ctx_id;
-}
-
-static int context_destroy(int fd, uint32_t ctx_id)
-{
-	int ret;
-	struct local_drm_i915_gem_context_destroy destroy;
-
-	destroy.ctx_id = ctx_id;
-	destroy.pad = rand();
-
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	if (ret != 0)
-		return -errno;
-
-	return 0;
-}
-
 static int gem_reset_stats(int fd, int ctx_id,
 			   struct local_drm_i915_reset_stats *rs)
 {
@@ -288,7 +243,7 @@ static int inject_hang_ring(int fd, int ctx, int ring, bool ignore_ban_error)
 	i915_execbuffer2_set_context_id(execbuf, ctx);
 	execbuf.rsvd2 = 0;
 
-	igt_assert(gem_exec(fd, &execbuf) == 0);
+	igt_assert_eq(gem_exec(fd, &execbuf), 0);
 
 	gtt_off = exec.offset;
 
@@ -329,7 +284,7 @@ static int inject_hang_ring(int fd, int ctx, int ring, bool ignore_ban_error)
 	i915_execbuffer2_set_context_id(execbuf, ctx);
 	execbuf.rsvd2 = 0;
 
-	igt_assert(gem_exec(fd, &execbuf) == 0);
+	igt_assert_eq(gem_exec(fd, &execbuf), 0);
 
 	igt_assert(gtt_off == exec.offset);
 
@@ -387,8 +342,8 @@ static void test_rs(int num_fds, int hang_index, int rs_assumed_no_hang)
 	int fd[MAX_FD];
 	int h[MAX_FD];
 
-	igt_assert (num_fds <= MAX_FD);
-	igt_assert (hang_index < MAX_FD);
+	igt_assert_lte(num_fds, MAX_FD);
+	igt_assert_lt(hang_index, MAX_FD);
 
 	for (i = 0; i < num_fds; i++) {
 		fd[i] = drm_open_any();
@@ -436,11 +391,11 @@ static void test_rs_ctx(int num_fds, int num_ctx, int hang_index,
 	int h[MAX_FD][MAX_CTX];
 	int ctx[MAX_FD][MAX_CTX];
 
-	igt_assert (num_fds <= MAX_FD);
-	igt_assert (hang_index < MAX_FD);
+	igt_assert_lte(num_fds, MAX_FD);
+	igt_assert_lt(hang_index, MAX_FD);
 
-	igt_assert (num_ctx <= MAX_CTX);
-	igt_assert (hang_context < MAX_CTX);
+	igt_assert_lte(num_ctx, MAX_CTX);
+	igt_assert_lt(hang_context, MAX_CTX);
 
 	test_rs(num_fds, -1, RS_NO_ERROR);
 
@@ -450,7 +405,7 @@ static void test_rs_ctx(int num_fds, int num_ctx, int hang_index,
 		assert_reset_status(fd[i], 0, RS_NO_ERROR);
 
 		for (j = 0; j < num_ctx; j++) {
-			ctx[i][j] = context_create(fd[i]);
+			ctx[i][j] = gem_context_create(fd[i]);
 
 		}
 
@@ -502,7 +457,7 @@ static void test_rs_ctx(int num_fds, int num_ctx, int hang_index,
 	for (i = 0; i < num_fds; i++) {
 		for (j = 0; j < num_ctx; j++) {
 			gem_close(fd[i], h[i][j]);
-			igt_assert(context_destroy(fd[i], ctx[i][j]) == 0);
+			gem_context_destroy(fd[i], ctx[i][j]);
 		}
 
 		assert_reset_status(fd[i], 0, RS_NO_ERROR);
@@ -520,24 +475,22 @@ static void test_ban(void)
 	struct local_drm_i915_reset_stats rs_bad, rs_good;
 
 	fd_bad = drm_open_any();
-	igt_assert(fd_bad >= 0);
 
 	fd_good = drm_open_any();
-	igt_assert(fd_good >= 0);
 
 	assert_reset_status(fd_bad, 0, RS_NO_ERROR);
 	assert_reset_status(fd_good, 0, RS_NO_ERROR);
 
 	h1 = exec_valid(fd_bad, 0);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	h5 = exec_valid(fd_good, 0);
-	igt_assert(h5 >= 0);
+	igt_assert_lte(0, h5);
 
 	assert_reset_status(fd_bad, 0, RS_NO_ERROR);
 	assert_reset_status(fd_good, 0, RS_NO_ERROR);
 
 	h2 = inject_hang_no_ban_error(fd_bad, 0);
-	igt_assert(h2 >= 0);
+	igt_assert_lte(0, h2);
 	active_count++;
 	/* Second hang will be pending for this */
 	pending_count++;
@@ -547,7 +500,7 @@ static void test_ban(void)
 
         while (retry--) {
                 h3 = inject_hang_no_ban_error(fd_bad, 0);
-                igt_assert(h3 >= 0);
+                igt_assert_lte(0, h3);
                 gem_sync(fd_bad, h3);
 		active_count++;
 		/* This second hand will count as pending */
@@ -562,21 +515,21 @@ static void test_ban(void)
                 /* Should not happen often but sometimes hang is declared too slow
                  * due to our way of faking hang using loop */
 
-                igt_assert(h4 >= 0);
+                igt_assert_lte(0, h4);
                 gem_close(fd_bad, h3);
                 gem_close(fd_bad, h4);
 
                 igt_info("retrying for ban (%d)\n", retry);
         }
 
-	igt_assert(h4 == -EIO);
+	igt_assert_eq(h4, -EIO);
 	assert_reset_status(fd_bad, 0, RS_BATCH_ACTIVE);
 
 	gem_sync(fd_good, h7);
 	assert_reset_status(fd_good, 0, RS_BATCH_PENDING);
 
-	igt_assert(gem_reset_stats(fd_good, 0, &rs_good) == 0);
-	igt_assert(gem_reset_stats(fd_bad, 0, &rs_bad) == 0);
+	igt_assert_eq(gem_reset_stats(fd_good, 0, &rs_good), 0);
+	igt_assert_eq(gem_reset_stats(fd_bad, 0, &rs_bad), 0);
 
 	igt_assert(rs_bad.batch_active == active_count);
 	igt_assert(rs_bad.batch_pending == pending_count);
@@ -589,14 +542,14 @@ static void test_ban(void)
 	gem_close(fd_good, h7);
 
 	h1 = exec_valid(fd_good, 0);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	gem_close(fd_good, h1);
 
 	close(fd_bad);
 	close(fd_good);
 
-	igt_assert(gem_reset_status(fd_bad, 0) < 0);
-	igt_assert(gem_reset_status(fd_good, 0) < 0);
+	igt_assert_lt(gem_reset_status(fd_bad, 0), 0);
+	igt_assert_lt(gem_reset_status(fd_good, 0), 0);
 }
 
 static void test_ban_ctx(void)
@@ -609,27 +562,26 @@ static void test_ban_ctx(void)
 	struct local_drm_i915_reset_stats rs_bad, rs_good;
 
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 
-	ctx_good = context_create(fd);
-	ctx_bad = context_create(fd);
+	ctx_good = gem_context_create(fd);
+	ctx_bad = gem_context_create(fd);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 	assert_reset_status(fd, ctx_good, RS_NO_ERROR);
 	assert_reset_status(fd, ctx_bad, RS_NO_ERROR);
 
 	h1 = exec_valid(fd, ctx_bad);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	h5 = exec_valid(fd, ctx_good);
-	igt_assert(h5 >= 0);
+	igt_assert_lte(0, h5);
 
 	assert_reset_status(fd, ctx_good, RS_NO_ERROR);
 	assert_reset_status(fd, ctx_bad, RS_NO_ERROR);
 
 	h2 = inject_hang_no_ban_error(fd, ctx_bad);
-	igt_assert(h2 >= 0);
+	igt_assert_lte(0, h2);
 	active_count++;
 	/* Second hang will be pending for this */
 	pending_count++;
@@ -639,7 +591,7 @@ static void test_ban_ctx(void)
 
         while (retry--) {
                 h3 = inject_hang_no_ban_error(fd, ctx_bad);
-                igt_assert(h3 >= 0);
+                igt_assert_lte(0, h3);
                 gem_sync(fd, h3);
 		active_count++;
 		/* This second hand will count as pending */
@@ -654,21 +606,21 @@ static void test_ban_ctx(void)
                 /* Should not happen often but sometimes hang is declared too slow
                  * due to our way of faking hang using loop */
 
-                igt_assert(h4 >= 0);
+                igt_assert_lte(0, h4);
                 gem_close(fd, h3);
                 gem_close(fd, h4);
 
                 igt_info("retrying for ban (%d)\n", retry);
         }
 
-	igt_assert(h4 == -EIO);
+	igt_assert_eq(h4, -EIO);
 	assert_reset_status(fd, ctx_bad, RS_BATCH_ACTIVE);
 
 	gem_sync(fd, h7);
 	assert_reset_status(fd, ctx_good, RS_BATCH_PENDING);
 
-	igt_assert(gem_reset_stats(fd, ctx_good, &rs_good) == 0);
-	igt_assert(gem_reset_stats(fd, ctx_bad, &rs_bad) == 0);
+	igt_assert_eq(gem_reset_stats(fd, ctx_good, &rs_good), 0);
+	igt_assert_eq(gem_reset_stats(fd, ctx_bad, &rs_bad), 0);
 
 	igt_assert(rs_bad.batch_active == active_count);
 	igt_assert(rs_bad.batch_pending == pending_count);
@@ -681,15 +633,15 @@ static void test_ban_ctx(void)
 	gem_close(fd, h7);
 
 	h1 = exec_valid(fd, ctx_good);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	gem_close(fd, h1);
 
-	igt_assert(context_destroy(fd, ctx_good) == 0);
-	igt_assert(context_destroy(fd, ctx_bad) == 0);
-	igt_assert(gem_reset_status(fd, ctx_good) < 0);
-	igt_assert(gem_reset_status(fd, ctx_bad) < 0);
-	igt_assert(exec_valid(fd, ctx_good) < 0);
-	igt_assert(exec_valid(fd, ctx_bad) < 0);
+	gem_context_destroy(fd, ctx_good);
+	gem_context_destroy(fd, ctx_bad);
+	igt_assert_lt(gem_reset_status(fd, ctx_good), 0);
+	igt_assert_lt(gem_reset_status(fd, ctx_bad), 0);
+	igt_assert_lt(exec_valid(fd, ctx_good), 0);
+	igt_assert_lt(exec_valid(fd, ctx_bad), 0);
 
 	close(fd);
 }
@@ -704,28 +656,28 @@ static void test_unrelated_ctx(void)
 	fd2 = drm_open_any();
 	assert_reset_status(fd1, 0, RS_NO_ERROR);
 	assert_reset_status(fd2, 0, RS_NO_ERROR);
-	ctx_guilty = context_create(fd1);
-	ctx_unrelated = context_create(fd2);
+	ctx_guilty = gem_context_create(fd1);
+	ctx_unrelated = gem_context_create(fd2);
 
 	assert_reset_status(fd1, ctx_guilty, RS_NO_ERROR);
 	assert_reset_status(fd2, ctx_unrelated, RS_NO_ERROR);
 
 	h1 = inject_hang(fd1, ctx_guilty);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	gem_sync(fd1, h1);
 	assert_reset_status(fd1, ctx_guilty, RS_BATCH_ACTIVE);
 	assert_reset_status(fd2, ctx_unrelated, RS_NO_ERROR);
 
 	h2 = exec_valid(fd2, ctx_unrelated);
-	igt_assert(h2 >= 0);
+	igt_assert_lte(0, h2);
 	gem_sync(fd2, h2);
 	assert_reset_status(fd1, ctx_guilty, RS_BATCH_ACTIVE);
 	assert_reset_status(fd2, ctx_unrelated, RS_NO_ERROR);
 	gem_close(fd1, h1);
 	gem_close(fd2, h2);
 
-	igt_assert(context_destroy(fd1, ctx_guilty) == 0);
-	igt_assert(context_destroy(fd2, ctx_unrelated) == 0);
+	gem_context_destroy(fd1, ctx_guilty);
+	gem_context_destroy(fd2, ctx_unrelated);
 
 	close(fd1);
 	close(fd2);
@@ -749,15 +701,14 @@ static void test_close_pending_ctx(void)
 	uint32_t ctx;
 
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
-	ctx = context_create(fd);
+	ctx = gem_context_create(fd);
 
 	assert_reset_status(fd, ctx, RS_NO_ERROR);
 
 	h = inject_hang(fd, ctx);
-	igt_assert(h >= 0);
-	igt_assert(context_destroy(fd, ctx) == 0);
-	igt_assert(context_destroy(fd, ctx) == -ENOENT);
+	igt_assert_lte(0, h);
+	gem_context_destroy(fd, ctx);
+	igt_assert(__gem_context_destroy(fd, ctx) == -ENOENT);
 
 	gem_close(fd, h);
 	close(fd);
@@ -768,12 +719,11 @@ static void test_close_pending(void)
 	int fd, h;
 
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 
 	h = inject_hang(fd, 0);
-	igt_assert(h >= 0);
+	igt_assert_lte(0, h);
 
 	gem_close(fd, h);
 	close(fd);
@@ -831,12 +781,11 @@ static void test_close_pending_fork(const bool reverse)
 	int fd, h;
 
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 
 	h = inject_hang(fd, 0);
-	igt_assert(h >= 0);
+	igt_assert_lte(0, h);
 
 	sleep(1);
 
@@ -847,7 +796,7 @@ static void test_close_pending_fork(const bool reverse)
 	pid = fork();
 	if (pid == 0) {
 		const int fd2 = drm_open_any();
-		igt_assert(fd2 >= 0);
+		igt_assert_lte(0, fd2);
 
 		/* The crucial component is that we schedule the same noop batch
 		 * on each ring. This exercises batch_obj reference counting,
@@ -858,7 +807,7 @@ static void test_close_pending_fork(const bool reverse)
 		close(fd2);
 		return;
 	} else {
-		igt_assert(pid > 0);
+		igt_assert_lt(0, pid);
 		sleep(1);
 
 		/* Kill the child to reduce refcounts on
@@ -871,10 +820,9 @@ static void test_close_pending_fork(const bool reverse)
 
 	/* Then we just wait on hang to happen */
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
 
 	h = exec_valid(fd, 0);
-	igt_assert(h >= 0);
+	igt_assert_lte(0, h);
 
 	gem_sync(fd, h);
 	gem_close(fd, h);
@@ -887,9 +835,8 @@ static void test_reset_count(const bool create_ctx)
 	long c1, c2;
 
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
 	if (create_ctx)
-		ctx = context_create(fd);
+		ctx = gem_context_create(fd);
 	else
 		ctx = 0;
 
@@ -899,7 +846,7 @@ static void test_reset_count(const bool create_ctx)
 	igt_assert(c1 >= 0);
 
 	h = inject_hang(fd, ctx);
-	igt_assert (h >= 0);
+	igt_assert_lte(0, h);
 	gem_sync(fd, h);
 
 	assert_reset_status(fd, ctx, RS_BATCH_ACTIVE);
@@ -923,7 +870,7 @@ static void test_reset_count(const bool create_ctx)
 	gem_close(fd, h);
 
 	if (create_ctx)
-		context_destroy(fd, ctx);
+		gem_context_destroy(fd, ctx);
 
 	close(fd);
 }
@@ -958,20 +905,20 @@ static void _check_param_ctx(const int fd, const int ctx, const cap_t cap)
 
 	if (ctx == 0) {
 		if (cap == root)
-			igt_assert(_test_params(fd, ctx, 0, 0) == 0);
+			igt_assert_eq(_test_params(fd, ctx, 0, 0), 0);
 		else
-			igt_assert(_test_params(fd, ctx, 0, 0) == -EPERM);
+			igt_assert_eq(_test_params(fd, ctx, 0, 0), -EPERM);
 	}
 
-	igt_assert(_test_params(fd, ctx, 0, bad) == -EINVAL);
-	igt_assert(_test_params(fd, ctx, bad, 0) == -EINVAL);
-	igt_assert(_test_params(fd, ctx, bad, bad) == -EINVAL);
+	igt_assert_eq(_test_params(fd, ctx, 0, bad), -EINVAL);
+	igt_assert_eq(_test_params(fd, ctx, bad, 0), -EINVAL);
+	igt_assert_eq(_test_params(fd, ctx, bad, bad), -EINVAL);
 }
 
 static void check_params(const int fd, const int ctx, cap_t cap)
 {
 	igt_assert(ioctl(fd, GET_RESET_STATS_IOCTL, 0) == -1);
-	igt_assert(_test_params(fd, 0xbadbad, 0, 0) == -ENOENT);
+	igt_assert_eq(_test_params(fd, 0xbadbad, 0, 0), -ENOENT);
 
 	_check_param_ctx(fd, ctx, cap);
 }
@@ -998,8 +945,7 @@ static void test_params_ctx(void)
 	int fd, ctx;
 
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
-	ctx = context_create(fd);
+	ctx = gem_context_create(fd);
 
 	_test_param(fd, ctx);
 
@@ -1011,7 +957,6 @@ static void test_params(void)
 	int fd;
 
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
 
 	_test_param(fd, 0);
 
@@ -1019,20 +964,42 @@ static void test_params(void)
 
 }
 
-static bool gem_has_hw_contexts(int fd)
+static void defer_hangcheck(int ring_num)
 {
-	struct local_drm_i915_gem_context_create create;
-	int ret;
+	int fd, count_start, count_end;
+	int seconds = 30;
+	const struct target_ring *next_ring;
+	fd = drm_open_any();
 
-	memset(&create, 0, sizeof(create));
-	ret = drmIoctl(fd, CONTEXT_CREATE_IOCTL, &create);
+	do {
+		next_ring = &rings[(++ring_num) % NUM_RINGS];
 
-	if (ret == 0) {
-		drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &create);
-		return true;
+		if (next_ring->present(fd))
+			break;
+
+	} while(next_ring != current_ring);
+
+	igt_skip_on(next_ring == current_ring);
+
+	count_start = get_reset_count(fd, 0);
+	igt_assert_lte(0, count_start);
+
+	igt_assert(inject_hang_ring(fd, 0, current_ring->exec, true));
+	while (--seconds) {
+		igt_assert(exec_valid_ring(fd, 0, next_ring->exec));
+
+		count_end = get_reset_count(fd, 0);
+		igt_assert_lte(0, count_end);
+
+		if (count_end > count_start)
+			break;
+
+		sleep(1);
 	}
 
-	return false;
+	igt_assert_lt(count_start, count_end);
+
+	close(fd);
 }
 
 static bool gem_has_reset_stats(int fd)
@@ -1081,9 +1048,8 @@ static void check_gpu_ok(void)
 	close(fd);
 }
 
-#define RING_HAS_CONTEXTS (current_ring->contexts(current_ring))
 #define RUN_TEST(...) do { check_gpu_ok(); __VA_ARGS__; check_gpu_ok(); } while (0)
-#define RUN_CTX_TEST(...) do { igt_skip_on(RING_HAS_CONTEXTS == false); RUN_TEST(__VA_ARGS__); } while (0)
+#define RUN_CTX_TEST(...) do { check_context(current_ring); RUN_TEST(__VA_ARGS__); } while (0)
 
 igt_main
 {
@@ -1096,7 +1062,6 @@ igt_main
 		fd = drm_open_any();
 		devid = intel_get_drm_devid(fd);
 
-		hw_contexts = gem_has_hw_contexts(fd);
 		has_reset_stats = gem_has_reset_stats(fd);
 
 		close(fd);
@@ -1159,5 +1124,9 @@ igt_main
 
 		igt_subtest_f("close-pending-fork-reverse-%s", name)
 			RUN_TEST(test_close_pending_fork(true));
+
+		igt_subtest_f("defer-hangcheck-%s", name)
+			RUN_TEST(defer_hangcheck(i));
+
 	}
 }
