@@ -32,6 +32,7 @@
 #include "drmtest.h"
 #include "igt_debugfs.h"
 #include "igt_kms.h"
+#include "igt_aux.h"
 
 typedef struct {
 	float red;
@@ -78,7 +79,7 @@ test_grab_crc(data_t *data, igt_output_t *output, enum pipe pipe,
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    I915_TILING_NONE,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    fb_color->red, fb_color->green, fb_color->blue,
 			    &fb);
 	igt_plane_set_fb(primary, &fb);
@@ -130,7 +131,7 @@ create_fb_for_mode__position(data_t *data, drmModeModeInfo *mode,
 	fb_id = igt_create_fb(data->drm_fd,
 				  mode->hdisplay, mode->vdisplay,
 				  DRM_FORMAT_XRGB8888,
-				  I915_TILING_NONE,
+				  LOCAL_DRM_FORMAT_MOD_NONE,
 				  fb);
 	igt_assert(fb_id);
 
@@ -144,6 +145,7 @@ create_fb_for_mode__position(data_t *data, drmModeModeInfo *mode,
 
 enum {
 	TEST_POSITION_FULLY_COVERED = 1 << 0,
+	TEST_DPMS = 1 << 1,
 };
 
 static void
@@ -157,7 +159,7 @@ test_plane_position_with_output(data_t *data,
 	igt_plane_t *primary, *sprite;
 	struct igt_fb primary_fb, sprite_fb;
 	drmModeModeInfo *mode;
-	igt_crc_t crc;
+	igt_crc_t crc, crc2;
 
 	igt_info("Testing connector %s using pipe %s plane %d\n",
 		 igt_output_name(output), kmstest_pipe_name(pipe), plane);
@@ -179,7 +181,7 @@ test_plane_position_with_output(data_t *data,
 	igt_create_color_fb(data->drm_fd,
 				64, 64, /* width, height */
 				DRM_FORMAT_XRGB8888,
-				I915_TILING_NONE,
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				0.0, 1.0, 0.0,
 				&sprite_fb);
 	igt_plane_set_fb(sprite, &sprite_fb);
@@ -193,11 +195,24 @@ test_plane_position_with_output(data_t *data,
 
 	igt_pipe_crc_collect_crc(data->pipe_crc, &crc);
 
+	if (flags & TEST_DPMS) {
+		kmstest_set_connector_dpms(data->drm_fd,
+					   output->config.connector,
+					   DRM_MODE_DPMS_OFF);
+		kmstest_set_connector_dpms(data->drm_fd,
+					   output->config.connector,
+					   DRM_MODE_DPMS_ON);
+	}
+
+	igt_pipe_crc_collect_crc(data->pipe_crc, &crc2);
+
 	if (flags & TEST_POSITION_FULLY_COVERED)
 		igt_assert(igt_crc_equal(&test.reference_crc, &crc));
 	else
 		igt_assert(!igt_crc_equal(&test.reference_crc, &crc));
 
+	igt_assert(igt_crc_equal(&crc, &crc2));
+
 	igt_plane_set_fb(primary, NULL);
 	igt_plane_set_fb(sprite, NULL);
 
@@ -248,7 +263,7 @@ create_fb_for_mode__panning(data_t *data, drmModeModeInfo *mode,
 	fb_id = igt_create_fb(data->drm_fd,
 			      mode->hdisplay * 2, mode->vdisplay * 2,
 			      DRM_FORMAT_XRGB8888,
-			      I915_TILING_NONE,
+			      LOCAL_DRM_FORMAT_MOD_NONE,
 			      fb);
 	igt_assert(fb_id);
 
@@ -269,6 +284,7 @@ create_fb_for_mode__panning(data_t *data, drmModeModeInfo *mode,
 enum {
 	TEST_PANNING_TOP_LEFT	  = 1 << 0,
 	TEST_PANNING_BOTTOM_RIGHT = 1 << 1,
+	TEST_SUSPEND_RESUME	  = 1 << 2,
 };
 
 static void
@@ -309,8 +325,13 @@ test_plane_panning_with_output(data_t *data,
 
 	igt_display_commit(&data->display);
 
+	if (flags & TEST_SUSPEND_RESUME)
+		igt_system_suspend_autoresume();
+
 	igt_pipe_crc_collect_crc(data->pipe_crc, &crc);
 
+	igt_debug_wait_for_keypress("crc");
+
 	if (flags & TEST_PANNING_TOP_LEFT)
 		igt_assert(igt_crc_equal(&test.red_crc, &crc));
 	else
@@ -351,6 +372,11 @@ run_tests_for_pipe_plane(data_t *data, enum pipe pipe, enum igt_plane plane)
 		      kmstest_pipe_name(pipe), plane)
 		test_plane_position(data, pipe, plane, 0);
 
+	igt_subtest_f("plane-position-hole-dpms-pipe-%s-plane-%d",
+		      kmstest_pipe_name(pipe), plane)
+		test_plane_position(data, pipe, plane,
+				    TEST_DPMS);
+
 	igt_subtest_f("plane-panning-top-left-pipe-%s-plane-%d",
 		      kmstest_pipe_name(pipe), plane)
 		test_plane_panning(data, pipe, plane, TEST_PANNING_TOP_LEFT);
@@ -360,6 +386,11 @@ run_tests_for_pipe_plane(data_t *data, enum pipe pipe, enum igt_plane plane)
 		test_plane_panning(data, pipe, plane,
 				   TEST_PANNING_BOTTOM_RIGHT);
 
+	igt_subtest_f("plane-panning-bottom-right-suspend-pipe-%s-plane-%d",
+		      kmstest_pipe_name(pipe), plane)
+		test_plane_panning(data, pipe, plane,
+				   TEST_PANNING_BOTTOM_RIGHT |
+				   TEST_SUSPEND_RESUME);
 }
 
 static void
@@ -379,7 +410,7 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		data.drm_fd = drm_open_any();
+		data.drm_fd = drm_open_any_master();
 
 		kmstest_set_vt_graphics_mode();
 
@@ -387,7 +418,7 @@ igt_main
 		igt_display_init(&data.display, data.drm_fd);
 	}
 
-	for (int pipe = 0; pipe < 3; pipe++)
+	for (int pipe = 0; pipe < I915_MAX_PIPES; pipe++)
 		run_tests_for_pipe(&data, pipe);
 
 	igt_fixture {
