@@ -62,31 +62,28 @@ functional_test_init(functional_test_t *test, igt_output_t *output, enum pipe pi
 	drmModeModeInfo *mode;
 
 	test->pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-	igt_skip_on_f(!test->pipe_crc,
-		      "auto crc not supported on this connector with pipe %i\n", pipe);
-
 
 	igt_output_set_pipe(output, pipe);
 
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				0.0, 0.0, 0.0,
 				&test->black_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				0.0, 0.0, 1.0,
 				&test->blue_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				1.0, 1.0, 0.0,
 				&test->yellow_fb);
 	igt_create_color_fb(data->drm_fd, 100, 100,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				1.0, 0.0, 0.0,
 				&test->red_fb);
 
@@ -128,7 +125,6 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	functional_test_t test = { .data = data };
 	igt_display_t *display = &data->display;
 	igt_plane_t *primary, *sprite;
-	int ret;
 	int num_primary = 0, num_cursor = 0;
 	int i;
 
@@ -152,8 +148,8 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 		else if (display->pipes[pipe].planes[i].is_cursor)
 			num_cursor++;
 
-	igt_assert(num_primary == 1);
-	igt_assert(num_cursor <= 1);
+	igt_assert_eq(num_primary, 1);
+	igt_assert_lte(num_cursor, 1);
 
 	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
 	sprite = igt_output_get_plane(output, IGT_PLANE_2);
@@ -220,10 +216,9 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
 
 	/* Step 10: Enable crtc (fb = -1), take CRC (CRC 7) */
-	ret = drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id, -1,
-			     0, 0, &output->config.connector->connector_id,
-			     1, test.mode);
-	igt_assert(ret == 0);
+	igt_assert(drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id, -1,
+				  0, 0, &output->config.connector->connector_id,
+				  1, test.mode) == 0);
 	igt_pipe_crc_collect_crc(test.pipe_crc, &test.crc_7);
 
 	/* Step 11: Disable primary plane */
@@ -241,16 +236,15 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	igt_display_commit2(display, COMMIT_LEGACY);
 
 	/* Step 14: Universal API, set primary completely offscreen (CRC 9) */
-	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-			      output->config.crtc->crtc_id,
-			      test.blue_fb.fb_id, 0,
-			      9000, 9000,
-			      test.mode->hdisplay,
-			      test.mode->vdisplay,
-			      IGT_FIXED(0,0), IGT_FIXED(0,0),
-			      IGT_FIXED(test.mode->hdisplay,0),
-			      IGT_FIXED(test.mode->vdisplay,0));
-	igt_assert(ret == 0);
+	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+				   output->config.crtc->crtc_id,
+				   test.blue_fb.fb_id, 0,
+				   9000, 9000,
+				   test.mode->hdisplay,
+				   test.mode->vdisplay,
+				   IGT_FIXED(0,0), IGT_FIXED(0,0),
+				   IGT_FIXED(test.mode->hdisplay,0),
+				   IGT_FIXED(test.mode->vdisplay,0)) == 0);
 	igt_pipe_crc_collect_crc(test.pipe_crc, &test.crc_9);
 
 	/*
@@ -316,19 +310,19 @@ sanity_test_init(sanity_test_t *test, igt_output_t *output, enum pipe pipe)
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->blue_fb);
 	igt_create_color_fb(data->drm_fd,
 			    mode->hdisplay + 100, mode->vdisplay + 100,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->oversized_fb);
 	igt_create_color_fb(data->drm_fd,
 			    mode->hdisplay - 100, mode->vdisplay - 100,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->undersized_fb);
 
@@ -360,7 +354,9 @@ sanity_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	sanity_test_t test = { .data = data };
 	igt_plane_t *primary;
 	drmModeModeInfo *mode;
-	int i, ret = 0;
+	int i;
+
+	igt_skip_on(pipe >= data->display.n_pipes);
 
 	igt_output_set_pipe(output, pipe);
 	mode = igt_output_get_mode(output);
@@ -378,54 +374,49 @@ sanity_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	 * doesn't cover CRTC (should fail).
 	 */
 	igt_plane_set_fb(primary, &test.undersized_fb);
-	ret = igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL);
-	igt_assert(ret == -EINVAL);
+	igt_assert(igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL) == -EINVAL);
 
 	/* Same as above, but different plane positioning. */
 	igt_plane_set_position(primary, 100, 100);
-	ret = igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL);
-	igt_assert(ret == -EINVAL);
+	igt_assert(igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL) == -EINVAL);
 
 	igt_plane_set_position(primary, 0, 0);
 
 	/* Try to use universal plane API to scale down (should fail) */
-	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-			      output->config.crtc->crtc_id,
-			      test.oversized_fb.fb_id, 0,
-			      0, 0,
-			      mode->hdisplay + 100,
-			      mode->vdisplay + 100,
-			      IGT_FIXED(0,0), IGT_FIXED(0,0),
-			      IGT_FIXED(mode->hdisplay,0),
-			      IGT_FIXED(mode->vdisplay,0));
-	igt_assert(ret == -ERANGE);
+	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+				   output->config.crtc->crtc_id,
+				   test.oversized_fb.fb_id, 0,
+				   0, 0,
+				   mode->hdisplay + 100,
+				   mode->vdisplay + 100,
+				   IGT_FIXED(0,0), IGT_FIXED(0,0),
+				   IGT_FIXED(mode->hdisplay,0),
+				   IGT_FIXED(mode->vdisplay,0)) == -ERANGE);
 
 	/* Try to use universal plane API to scale up (should fail) */
-	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-			      output->config.crtc->crtc_id,
-			      test.oversized_fb.fb_id, 0,
-			      0, 0,
-			      mode->hdisplay,
-			      mode->vdisplay,
-			      IGT_FIXED(0,0), IGT_FIXED(0,0),
-			      IGT_FIXED(mode->hdisplay - 100,0),
-			      IGT_FIXED(mode->vdisplay - 100,0));
-	igt_assert(ret == -ERANGE);
+	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+				   output->config.crtc->crtc_id,
+				   test.oversized_fb.fb_id, 0,
+				   0, 0,
+				   mode->hdisplay,
+				   mode->vdisplay,
+				   IGT_FIXED(0,0), IGT_FIXED(0,0),
+				   IGT_FIXED(mode->hdisplay - 100,0),
+				   IGT_FIXED(mode->vdisplay - 100,0)) == -ERANGE);
 
 	/* Find other crtcs and try to program our primary plane on them */
 	for (i = 0; i < test.moderes->count_crtcs; i++)
 		if (test.moderes->crtcs[i] != output->config.crtc->crtc_id) {
-			ret = drmModeSetPlane(data->drm_fd,
-					      primary->drm_plane->plane_id,
-					      test.moderes->crtcs[i],
-					      test.blue_fb.fb_id, 0,
-					      0, 0,
-					      mode->hdisplay,
-					      mode->vdisplay,
-					      IGT_FIXED(0,0), IGT_FIXED(0,0),
-					      IGT_FIXED(mode->hdisplay,0),
-					      IGT_FIXED(mode->vdisplay,0));
-			igt_assert(ret == -EINVAL);
+			igt_assert(drmModeSetPlane(data->drm_fd,
+						   primary->drm_plane->plane_id,
+						   test.moderes->crtcs[i],
+						   test.blue_fb.fb_id, 0,
+						   0, 0,
+						   mode->hdisplay,
+						   mode->vdisplay,
+						   IGT_FIXED(0,0), IGT_FIXED(0,0),
+						   IGT_FIXED(mode->hdisplay,0),
+						   IGT_FIXED(mode->vdisplay,0)) == -EINVAL);
 		}
 
 	igt_plane_set_fb(primary, NULL);
@@ -443,12 +434,12 @@ pageflip_test_init(pageflip_test_t *test, igt_output_t *output, enum pipe pipe)
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    1.0, 0.0, 0.0,
 			    &test->red_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->blue_fb);
 }
@@ -474,6 +465,8 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	fd_set fds;
 	int ret = 0;
 
+	igt_skip_on(pipe >= data->display.n_pipes);
+
 	igt_output_set_pipe(output, pipe);
 
 	pageflip_test_init(&test, output, pipe);
@@ -494,9 +487,8 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	 * Note that crtc->primary->fb = NULL causes flip to return EBUSY for
 	 * historical reasons...
 	 */
-	ret = drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
-			      test.red_fb.fb_id, 0, NULL);
-	igt_assert(ret == -EBUSY);
+	igt_assert(drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
+				   test.red_fb.fb_id, 0, NULL) == -EBUSY);
 
 	/* Turn primary plane back on */
 	igt_plane_set_fb(primary, &test.blue_fb);
@@ -509,10 +501,9 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	 * completes, which we don't have a good way to specifically test for,
 	 * but at least we can make sure that nothing blows up.
 	 */
-	ret = drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
-			      test.red_fb.fb_id, DRM_MODE_PAGE_FLIP_EVENT,
-			      &test);
-	igt_assert(ret == 0);
+	igt_assert(drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
+				   test.red_fb.fb_id, DRM_MODE_PAGE_FLIP_EVENT,
+				   &test) == 0);
 	igt_plane_set_fb(primary, NULL);
 	igt_display_commit2(&data->display, COMMIT_UNIVERSAL);
 
@@ -522,14 +513,126 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	do {
 		ret = select(data->drm_fd + 1, &fds, NULL, NULL, &timeout);
 	} while (ret < 0 && errno == EINTR);
-	igt_assert(ret == 1);
-	ret = drmHandleEvent(data->drm_fd, &evctx);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 1);
+	igt_assert(drmHandleEvent(data->drm_fd, &evctx) == 0);
 
 	igt_plane_set_fb(primary, NULL);
 	pageflip_test_fini(&test, output);
 }
 
+static void
+cursor_leak_test_fini(data_t *data,
+		      igt_output_t *output,
+		      struct igt_fb *bg,
+		      struct igt_fb *curs)
+{
+	int i;
+
+	igt_remove_fb(data->drm_fd, bg);
+	for (i = 0; i < 10; i++)
+		igt_remove_fb(data->drm_fd, &curs[i]);
+
+	igt_output_set_pipe(output, PIPE_ANY);
+}
+
+static int
+i915_gem_fb_count(void)
+{
+	char buf[1024];
+	FILE *fp;
+	int count = 0;
+
+	fp = igt_debugfs_fopen("i915_gem_framebuffer", "r");
+	igt_require(fp);
+	while (fgets(buf, sizeof(buf), fp) != NULL)
+		count++;
+	fclose(fp);
+
+	return count;
+}
+
+static void
+cursor_leak_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
+{
+	igt_display_t *display = &data->display;
+	igt_plane_t *primary, *cursor;
+	drmModeModeInfo *mode;
+	struct igt_fb background_fb;
+	struct igt_fb cursor_fb[10];
+	int i;
+	int r, g, b;
+	int count1, count2;
+
+	igt_assert(data->display.has_universal_planes);
+	igt_skip_on(pipe >= display->n_pipes);
+
+	igt_output_set_pipe(output, pipe);
+	mode = igt_output_get_mode(output);
+
+	/* Count GEM framebuffers before creating our cursor FB's */
+	count1 = i915_gem_fb_count();
+
+	/* Black background FB */
+	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
+			    DRM_FORMAT_XRGB8888,
+			    false,
+			    0.0, 0.0, 0.0,
+			    &background_fb);
+
+	/* Random color cursors */
+	for (i = 0; i < 10; i++) {
+		r = rand() % 0xFF;
+		g = rand() % 0xFF;
+		b = rand() % 0xFF;
+		igt_create_color_fb(data->drm_fd, 64, 64,
+				    DRM_FORMAT_ARGB8888,
+				    false,
+				    (double)r / 0xFF,
+				    (double)g / 0xFF,
+				    (double)b / 0xFF,
+				    &cursor_fb[i]);
+	}
+
+	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+	cursor = igt_output_get_plane(output, IGT_PLANE_CURSOR);
+	if (!primary || !cursor) {
+		cursor_leak_test_fini(data, output, &background_fb, cursor_fb);
+		igt_skip("Primary and/or cursor are unavailable\n");
+	}
+
+
+	igt_plane_set_fb(primary, &background_fb);
+	igt_display_commit2(display, COMMIT_LEGACY);
+
+	igt_plane_set_position(cursor, 100, 100);
+
+	/*
+	 * Exercise both legacy and universal code paths.  Note that legacy
+	 * handling in the kernel redirects through universal codepaths
+	 * internally, so that redirection is where we're most worried about
+	 * leaking.
+	 */
+	for (i = 0; i < 10; i++) {
+		igt_plane_set_fb(cursor, &cursor_fb[i]);
+		igt_display_commit2(display, COMMIT_UNIVERSAL);
+	}
+	for (i = 0; i < 10; i++) {
+		igt_plane_set_fb(cursor, &cursor_fb[i]);
+		igt_display_commit2(display, COMMIT_LEGACY);
+	}
+
+	/* Release our framebuffer handles before we take a second count */
+	igt_plane_set_fb(primary, NULL);
+	igt_plane_set_fb(cursor, NULL);
+	igt_display_commit2(display, COMMIT_LEGACY);
+	cursor_leak_test_fini(data, output, &background_fb, cursor_fb);
+
+	/* We should be back to the same framebuffer count as when we started */
+	count2 = i915_gem_fb_count();
+
+	igt_assert_eq(count1, count2);
+}
+
 static void
 run_tests_for_pipe(data_t *data, enum pipe pipe)
 {
@@ -549,18 +652,21 @@ run_tests_for_pipe(data_t *data, enum pipe pipe)
 		      kmstest_pipe_name(pipe))
 		for_each_connected_output(&data->display, output)
 			pageflip_test_pipe(data, pipe, output);
+
+	igt_subtest_f("cursor-fb-leak-pipe-%s",
+		      kmstest_pipe_name(pipe))
+		for_each_connected_output(&data->display, output)
+			cursor_leak_test_pipe(data, pipe, output);
 }
 
 static data_t data;
 
 igt_main
 {
-	int num_pipes;
-
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		data.drm_fd = drm_open_any();
+		data.drm_fd = drm_open_any_master();
 
 		kmstest_set_vt_graphics_mode();
 
@@ -570,8 +676,7 @@ igt_main
 		igt_require(data.display.has_universal_planes);
 	}
 
-	num_pipes = igt_display_get_n_pipes(&data.display);
-	for (int pipe = 0; pipe < num_pipes; pipe++)
+	for (int pipe = 0; pipe < I915_MAX_PIPES; pipe++)
 		run_tests_for_pipe(&data, pipe);
 
 	igt_fixture {
