@@ -48,6 +48,9 @@
 #include "drmtest.h"
 #include "igt_debugfs.h"
 
+IGT_TEST_DESCRIPTION("Check whether prime import/export works on the same"
+		     " device.");
+
 #define BO_SIZE (16*1024)
 
 static char counter;
@@ -159,7 +162,7 @@ static void test_with_one_bo_two_files(void)
 	handle_import = prime_fd_to_handle(fd2, dma_buf_fd2);
 
 	/* dma-buf selfimporting an flink bo should give the same handle */
-	igt_assert(handle_import == handle_open);
+	igt_assert_eq_u32(handle_import, handle_open);
 
 	close(fd1);
 	close(fd2);
@@ -186,11 +189,11 @@ static void test_with_one_bo(void)
 	/* reimport should give us the same handle so that userspace can check
 	 * whether it has that bo already somewhere. */
 	handle_import2 = prime_fd_to_handle(fd2, dma_buf_fd);
-	igt_assert(handle_import1 == handle_import2);
+	igt_assert_eq_u32(handle_import1, handle_import2);
 
 	/* Same for re-importing on the exporting fd. */
 	handle_selfimport = prime_fd_to_handle(fd1, dma_buf_fd);
-	igt_assert(handle == handle_selfimport);
+	igt_assert_eq_u32(handle, handle_selfimport);
 
 	/* close dma_buf, check whether nothing disappears. */
 	close(dma_buf_fd);
@@ -216,18 +219,13 @@ static int get_object_count(void)
 {
 	FILE *file;
 	int ret, scanned;
-	int device = drm_get_card();
-	char *path;
 
 	igt_drop_caches_set(DROP_RETIRE);
 
-	ret = asprintf(&path, "/sys/kernel/debug/dri/%d/i915_gem_objects", device);
-	igt_assert(ret != -1);
-
-	file = fopen(path, "r");
+	file = igt_debugfs_fopen("i915_gem_objects", "r");
 
 	scanned = fscanf(file, "%i objects", &ret);
-	igt_assert(scanned == 1);
+	igt_assert_eq(scanned, 1);
 
 	return ret;
 }
@@ -271,7 +269,6 @@ static void test_reimport_close_race(void)
 	threads = calloc(num_threads, sizeof(pthread_t));
 
 	fds[0] = drm_open_any();
-	igt_assert(fds[0] >= 0);
 
 	handle = gem_create(fds[0], BO_SIZE);
 
@@ -281,7 +278,7 @@ static void test_reimport_close_race(void)
 		r = pthread_create(&threads[i], NULL,
 				   thread_fn_reimport_vs_close,
 				   (void *)(uintptr_t)fds);
-		igt_assert(r == 0);
+		igt_assert_eq(r, 0);
 	}
 
 	sleep(5);
@@ -345,21 +342,25 @@ static void test_export_close_race(void)
 	int fd;
 	int obj_count;
 	void *status;
-
-	obj_count = get_object_count();
+	int fake;
 
 	num_threads = sysconf(_SC_NPROCESSORS_ONLN);
 
 	threads = calloc(num_threads, sizeof(pthread_t));
 
+	/* Allocate exit handler fds in here so that we dont screw
+	 * up the counts */
+	fake = drm_open_any();
+
+	obj_count = get_object_count();
+
 	fd = drm_open_any();
-	igt_assert(fd >= 0);
 
 	for (i = 0; i < num_threads; i++) {
 		r = pthread_create(&threads[i], NULL,
 				   thread_fn_export_vs_close,
 				   (void *)(uintptr_t)fd);
-		igt_assert(r == 0);
+		igt_assert_eq(r, 0);
 	}
 
 	sleep(5);
@@ -376,6 +377,9 @@ static void test_export_close_race(void)
 	obj_count = get_object_count() - obj_count;
 
 	igt_info("leaked %i objects\n", obj_count);
+
+	close(fake);
+
 	igt_assert_eq(obj_count, 0);
 }
 
